import baselines.common.tf_util as U
import tensorflow as tf
import gym
from baselines.common.distributions import make_pdtype
from gibson.core.render.profiler import Profiler

class CnnPolicy(object):
    recurrent = False
    def __init__(self, name, ob_space, ac_space, session, save_per_acts=1e4, kind='large'):
        self.total_count = 0
        self.curr_count = 0
        self.save_per_acts = save_per_acts
        self.session = session
        with tf.variable_scope(name):
            self._init(ob_space, ac_space, kind)
            self.scope = tf.get_variable_scope().name

    def _init(self, ob_space, ac_space, kind):
        assert isinstance(ob_space, gym.spaces.Box)

        self.pdtype = pdtype = make_pdtype(ac_space)
        sequence_length = None

        ob = U.get_placeholder(name="ob", dtype=tf.float32, shape=[sequence_length] + list(ob_space.shape))
        print("CNNPolicy registering")

        x = ob / 255.0
        if kind == 'small': # from A3C paper
            x = tf.nn.relu(U.conv2d(x, 16, "l1", [8, 8], [4, 4], pad="VALID"))
            x = tf.nn.relu(U.conv2d(x, 32, "l2", [4, 4], [2, 2], pad="VALID"))
            x = U.flattenallbut0(x)
            x = tf.nn.relu(tf.layers.dense(x, 256, name='lin', kernel_initializer = U.normc_initializer(1.0)))
        elif kind == 'large': # Nature DQN
            x = tf.nn.relu(U.conv2d(x, 32, "l1", [8, 8], [4, 4], pad="VALID"))
            x = tf.nn.relu(U.conv2d(x, 64, "l2", [4, 4], [2, 2], pad="VALID"))
            x = tf.nn.relu(U.conv2d(x, 64, "l3", [3, 3], [1, 1], pad="VALID"))
            x = U.flattenallbut0(x)
            x = tf.nn.relu(tf.layers.dense(x, 512, name='lin', kernel_initializer=U.normc_initializer(1.0)))
        else:
            raise NotImplementedError

        ## Saver
        #self.saver = tf.train.Saver()


        logits = tf.layers.dense(x, pdtype.param_shape()[0], name="logits", kernel_initializer=U.normc_initializer(0.01))
        self.pd = pdtype.pdfromflat(logits)
        self.vpred = tf.layers.dense(x, 1, name="value", kernel_initializer=U.normc_initializer(1.0))[:,0]

        self.state_in = []
        self.state_out = []

        stochastic = tf.placeholder(dtype=tf.bool, shape=())
        ac = self.pd.sample() # XXX
        self._act = U.function([stochastic, ob], [ac, self.vpred])


    def act(self, stochastic, ob):
        ac1, vpred1 =  self._act(stochastic, ob[None])
        self.total_count = self.total_count + 1
        self.curr_count = self.curr_count + 1
        #if self.curr_count > self.save_per_acts:
        #    self.curr_count = self.curr_count - self.save_per_acts
        #    self.saver.save(self.session, 'cnn_policy',  global_step=self.total_count)
        return ac1[0], vpred1[0]
    def get_variables(self):
        return tf.get_collection(tf.GraphKeys.GLOBAL_VARIABLES, self.scope)
    def get_trainable_variables(self):
        return tf.get_collection(tf.GraphKeys.TRAINABLE_VARIABLES, self.scope)
    def get_initial_state(self):
        return []




def get_segmentId_by_name_MP3D(meta, name):
    category_ids = []
    object_ids = []
    segment_ids = []
    with open(meta) as f:
        for line in f:
            if line[:2] == "C " and name in line:
                c_index = int(line.split()[1])
                category_ids.append(c_index)
            if line[:2] == "O ":
                c_index = int(line.split()[3])
                o_index = int(line.split()[1])
                if c_index in category_ids:
                    object_ids.append(o_index)
            if line[:2] == "E ":
                o_index = int(line.split()[2])
                e_index = int(line.split()[1])
                if o_index in object_ids:
                    segment_ids.append(e_index)
    return category_ids, object_ids, segment_ids


def get_segmentId_by_name_2D3DS(mtl, obj, name):
    semantic_label_list = get_semantic_label_list_2D3DS(obj)
    object_ids = []
    obj_id = 0
    '''with open(mtl) as f:
        for line in f:
            if "newmtl" in line:
                semantic_label = line.split()[1]
                object_name, object_counchairt, segment_count = semantic_label.split("_")
                if name in object_name:
                    object_ids.append(obj_id)
                obj_id += 1
    '''
    for semantic_label in semantic_label_list:
        assert len(semantic_label.split("_")) == 5 or len(semantic_label.split("_")) == 3, "Unable to parse semantic label {}".format(semantic_label)
        if len(semantic_label.split("_")) == 5:
            object_name, object_count, context_name, context_count, floor_count = semantic_label.split("_")
            if name in object_name:
                object_ids.append(obj_id)
            obj_id += 1 
        if len(semantic_label.split("_")) == 3:
            object_name, object_count, floor_count = semantic_label.split("_")
            if name in object_name:
                object_ids.append(obj_id)
            obj_id += 1
    return None, object_ids, None


def get_semantic_label_list_2D3DS(obj):
    label_list = []
    with open(obj) as f:
        for line in f:
            if "usemtl " in line:
                label_list.append(line.split()[1])
    return label_list

if __name__ == "__main__":
    meta = ""
    #meta = "/home/zhiyang/Desktop/universe-test/GibsonEnv/gibson/assets/dataset/17DRP5sb8fy/semantic.house"
    c_id, o_id, e_id = get_segmentId_by_name(meta, "chair")
    print(len(c_id))
    print(len(o_id))
    print(len(e_id))
    print(o_id)


# <hr>Calculates the tangents and bitangents for the imported meshes. 
#
# Does nothing if a mesh does not have normals. You might want this post 
# processing step to be executed if you plan to use tangent space calculations 
# such as normal mapping  applied to the meshes. There's a config setting,
# <tt>#AI_CONFIG_PP_CT_MAX_SMOOTHING_ANGLE<tt>, which allows you to specify
# a maximum smoothing angle for the algorithm. However, usually you'll
# want to leave it at the default value.
#
aiProcess_CalcTangentSpace = 0x1

## <hr>Identifies and joins identical vertex data sets within all 
#  imported meshes. 
# 
# After this step is run, each mesh contains unique vertices,
# so a vertex may be used by multiple faces. You usually want
# to use this post processing step. If your application deals with
# indexed geometry, this step is compulsory or you'll just waste rendering
# time. <b>If this flag is not specified<b>, no vertices are referenced by
# more than one face and <b>no index buffer is required<b> for rendering.
#
aiProcess_JoinIdenticalVertices = 0x2

## <hr>Converts all the imported data to a left-handed coordinate space. 
#
# By default the data is returned in a right-handed coordinate space (which 
# OpenGL prefers). In this space, +X points to the right, 
# +Z points towards the viewer, and +Y points upwards. In the DirectX 
# coordinate space +X points to the right, +Y points upwards, and +Z points 
# away from the viewer.
#
# You'll probably want to consider this flag if you use Direct3D for
# rendering. The #aiProcess_ConvertToLeftHanded flag supersedes this
# setting and bundles all conversions typically required for D3D-based
# applications.
#
aiProcess_MakeLeftHanded = 0x4

## <hr>Triangulates all faces of all meshes. 
#
# By default the imported mesh data might contain faces with more than 3
# indices. For rendering you'll usually want all faces to be triangles. 
# This post processing step splits up faces with more than 3 indices into 
# triangles. Line and point primitives are #not# modified! If you want
# 'triangles only' with no other kinds of primitives, try the following
# solution:
# <ul>
# <li>Specify both #aiProcess_Triangulate and #aiProcess_SortByPType <li>
# <li>Ignore all point and line meshes when you process assimp's output<li>
# <ul>
#
aiProcess_Triangulate = 0x8

## <hr>Removes some parts of the data structure (animations, materials, 
#  light sources, cameras, textures, vertex components).
#
# The  components to be removed are specified in a separate
# configuration option, <tt>#AI_CONFIG_PP_RVC_FLAGS<tt>. This is quite useful
# if you don't need all parts of the output structure. Vertex colors
# are rarely used today for example... Calling this step to remove unneeded
# data from the pipeline as early as possible results in increased 
# performance and a more optimized output data structure.
# This step is also useful if you want to force Assimp to recompute 
# normals or tangents. The corresponding steps don't recompute them if 
# they're already there (loaded from the source asset). By using this 
# step you can make sure they are NOT there.
#
# This flag is a poor one, mainly because its purpose is usually
# misunderstood. Consider the following case: a 3D model has been exported
# from a CAD app, and it has per-face vertex colors. Vertex positions can't be
# shared, thus the #aiProcess_JoinIdenticalVertices step fails to
# optimize the data because of these nasty little vertex colors.
# Most apps don't even process them, so it's all for nothing. By using
# this step, unneeded components are excluded as early as possible
# thus opening more room for internal optimizations. 
#
aiProcess_RemoveComponent = 0x10

## <hr>Generates normals for all faces of all meshes. 
#
# This is ignored if normals are already there at the time this flag 
# is evaluated. Model importers try to load them from the source file, so
# they're usually already there. Face normals are shared between all points
# of a single face, so a single point can have multiple normals, which
# forces the library to duplicate vertices in some cases.
# #aiProcess_JoinIdenticalVertices is #senseless# then.
#
# This flag may not be specified together with #aiProcess_GenSmoothNormals.
#
aiProcess_GenNormals = 0x20

## <hr>Generates smooth normals for all vertices in the mesh.
#
# This is ignored if normals are already there at the time this flag 
# is evaluated. Model importers try to load them from the source file, so
# they're usually already there. 
#
# This flag may not be specified together with 
# #aiProcess_GenNormals. There's a configuration option, 
# <tt>#AI_CONFIG_PP_GSN_MAX_SMOOTHING_ANGLE<tt> which allows you to specify
# an angle maximum for the normal smoothing algorithm. Normals exceeding
# this limit are not smoothed, resulting in a 'hard' seam between two faces.
# Using a decent angle here (e.g. 80 degrees) results in very good visual
# appearance.
#
aiProcess_GenSmoothNormals = 0x40

## <hr>Splits large meshes into smaller sub-meshes.
#
# This is quite useful for real-time rendering, where the number of triangles
# which can be maximally processed in a single draw-call is limited 
# by the video driverhardware. The maximum vertex buffer is usually limited
# too. Both requirements can be met with this step: you may specify both a 
# triangle and vertex limit for a single mesh.
#
# The split limits can (and should!) be set through the 
# <tt>#AI_CONFIG_PP_SLM_VERTEX_LIMIT<tt> and <tt>#AI_CONFIG_PP_SLM_TRIANGLE_LIMIT<tt> 
# settings. The default values are <tt>#AI_SLM_DEFAULT_MAX_VERTICES<tt> and 
# <tt>#AI_SLM_DEFAULT_MAX_TRIANGLES<tt>. 
#
# Note that splitting is generally a time-consuming task, but only if there's
# something to split. The use of this step is recommended for most users.
#
aiProcess_SplitLargeMeshes = 0x80

## <hr>Removes the node graph and pre-transforms all vertices with
# the local transformation matrices of their nodes.
#
# The output scene still contains nodes, however there is only a
# root node with children, each one referencing only one mesh,
# and each mesh referencing one material. For rendering, you can
# simply render all meshes in order - you don't need to pay
# attention to local transformations and the node hierarchy.
# Animations are removed during this step.
# This step is intended for applications without a scenegraph.
# The step CAN cause some problems: if e.g. a mesh of the asset
# contains normals and another, using the same material index, does not, 
# they will be brought together, but the first meshes's part of
# the normal list is zeroed. However, these artifacts are rare.
# @note The <tt>#AI_CONFIG_PP_PTV_NORMALIZE<tt> configuration property
# can be set to normalize the scene's spatial dimension to the -1...1
# range. 
#
aiProcess_PreTransformVertices = 0x100

## <hr>Limits the number of bones simultaneously affecting a single vertex
#  to a maximum value.
#
# If any vertex is affected by more than the maximum number of bones, the least 
# important vertex weights are removed and the remaining vertex weights are
# renormalized so that the weights still sum up to 1.
# The default bone weight limit is 4 (defined as <tt>#AI_LMW_MAX_WEIGHTS<tt> in
# config.h), but you can use the <tt>#AI_CONFIG_PP_LBW_MAX_WEIGHTS<tt> setting to
# supply your own limit to the post processing step.
#
# If you intend to perform the skinning in hardware, this post processing 
# step might be of interest to you.
#
aiProcess_LimitBoneWeights = 0x200

## <hr>Validates the imported scene data structure.
# This makes sure that all indices are valid, all animations and
# bones are linked correctly, all material references are correct .. etc.
#
# It is recommended that you capture Assimp's log output if you use this flag,
# so you can easily find out what's wrong if a file fails the
# validation. The validator is quite strict and will find #all#
# inconsistencies in the data structure... It is recommended that plugin 
# developers use it to debug their loaders. There are two types of
# validation failures:
# <ul>
# <li>Error: There's something wrong with the imported data. Further
#   postprocessing is not possible and the data is not usable at all.
#   The import fails. #Importer::GetErrorString() or #aiGetErrorString()
#   carry the error message around.<li>
# <li>Warning: There are some minor issues (e.g. 1000000 animation 
#   keyframes with the same time), but further postprocessing and use
#   of the data structure is still safe. Warning details are written
#   to the log file, <tt>#AI_SCENE_FLAGS_VALIDATION_WARNING<tt> is set
#   in #aiScene::mFlags<li>
# <ul>
#
# This post-processing step is not time-consuming. Its use is not
# compulsory, but recommended. 
#
aiProcess_ValidateDataStructure = 0x400

## <hr>Reorders triangles for better vertex cache locality.
#
# The step tries to improve the ACMR (average post-transform vertex cache
# miss ratio) for all meshes. The implementation runs in O(n) and is 
# roughly based on the 'tipsify' algorithm (see <a href="
# http:www.cs.princeton.edugfxpubsSander_2007_%3ETRtipsy.pdf">this
# paper<a>).
#
# If you intend to render huge models in hardware, this step might
# be of interest to you. The <tt>#AI_CONFIG_PP_ICL_PTCACHE_SIZE<tt>config
# setting can be used to fine-tune the cache optimization.
#
aiProcess_ImproveCacheLocality = 0x800

## <hr>Searches for redundantunreferenced materials and removes them.
#
# This is especially useful in combination with the 
# #aiProcess_PretransformVertices and #aiProcess_OptimizeMeshes flags. 
# Both join small meshes with equal characteristics, but they can't do 
# their work if two meshes have different materials. Because several
# material settings are lost during Assimp's import filters,
# (and because many exporters don't check for redundant materials), huge
# models often have materials which are are defined several times with
# exactly the same settings.
#
# Several material settings not contributing to the final appearance of
# a surface are ignored in all comparisons (e.g. the material name).
# So, if you're passing additional information through the
# content pipeline (probably using #magic# material names), don't 
# specify this flag. Alternatively take a look at the
# <tt>#AI_CONFIG_PP_RRM_EXCLUDE_LIST<tt> setting.
# 
aiProcess_RemoveRedundantMaterials = 0x1000

## <hr>This step tries to determine which meshes have normal vectors 
# that are facing inwards and inverts them.
#
# The algorithm is simple but effective:
# the bounding box of all vertices + their normals is compared against
# the volume of the bounding box of all vertices without their normals.
# This works well for most objects, problems might occur with planar
# surfaces. However, the step tries to filter such cases.
# The step inverts all in-facing normals. Generally it is recommended
# to enable this step, although the result is not always correct.
#
aiProcess_FixInfacingNormals = 0x2000

## <hr>This step splits meshes with more than one primitive type in 
#  homogeneous sub-meshes. 
#
#  The step is executed after the triangulation step. After the step
#  returns, just one bit is set in aiMesh::mPrimitiveTypes. This is 
#  especially useful for real-time rendering where point and line
#  primitives are often ignored or rendered separately.
#  You can use the <tt>#AI_CONFIG_PP_SBP_REMOVE<tt> option to specify which
#  primitive types you need. This can be used to easily exclude
#  lines and points, which are rarely used, from the import.
#
aiProcess_SortByPType = 0x8000

## <hr>This step searches all meshes for degenerate primitives and
#  converts them to proper lines or points.
#
# A face is 'degenerate' if one or more of its points are identical.
# To have the degenerate stuff not only detected and collapsed but
# removed, try one of the following procedures:
# <br><b>1.<b> (if you support lines and points for rendering but don't
#    want the degenerates)<br>
# <ul>
#   <li>Specify the #aiProcess_FindDegenerates flag.
#   <li>
#   <li>Set the <tt>AI_CONFIG_PP_FD_REMOVE<tt> option to 1. This will 
#       cause the step to remove degenerate triangles from the import
#       as soon as they're detected. They won't pass any further
#       pipeline steps.
#   <li>
# <ul>
# <br><b>2.<b>(if you don't support lines and points at all)<br>
# <ul>
#   <li>Specify the #aiProcess_FindDegenerates flag.
#   <li>
#   <li>Specify the #aiProcess_SortByPType flag. This moves line and
#     point primitives to separate meshes.
#   <li>
#   <li>Set the <tt>AI_CONFIG_PP_SBP_REMOVE<tt> option to 
#       @code aiPrimitiveType_POINTS | aiPrimitiveType_LINES
#       @endcode to cause SortByPType to reject point
#       and line meshes from the scene.
#   <li>
# <ul>
# @note Degenerate polygons are not necessarily evil and that's why
# they're not removed by default. There are several file formats which 
# don't support lines or points, and some exporters bypass the
# format specification and write them as degenerate triangles instead.
#
aiProcess_FindDegenerates = 0x10000

## <hr>This step searches all meshes for invalid data, such as zeroed
#  normal vectors or invalid UV coords and removesfixes them. This is
#  intended to get rid of some common exporter errors.
#
# This is especially useful for normals. If they are invalid, and
# the step recognizes this, they will be removed and can later
# be recomputed, i.e. by the #aiProcess_GenSmoothNormals flag.<br>
# The step will also remove meshes that are infinitely small and reduce
# animation tracks consisting of hundreds if redundant keys to a single
# key. The <tt>AI_CONFIG_PP_FID_ANIM_ACCURACY<tt> config property decides
# the accuracy of the check for duplicate animation tracks.
#
aiProcess_FindInvalidData = 0x20000

## <hr>This step converts non-UV mappings (such as spherical or
#  cylindrical mapping) to proper texture coordinate channels.
#
# Most applications will support UV mapping only, so you will
# probably want to specify this step in every case. Note that Assimp is not
# always able to match the original mapping implementation of the 
# 3D app which produced a model perfectly. It's always better to let the
# modelling app compute the UV channels - 3ds max, Maya, Blender, 
# LightWave, and Modo do this for example.
#
# @note If this step is not requested, you'll need to process the
# <tt>#AI_MATKEY_MAPPING<tt> material property in order to display all assets
# properly.
#
aiProcess_GenUVCoords = 0x40000

## <hr>This step applies per-texture UV transformations and bakes
#  them into stand-alone vtexture coordinate channels.
#  
# UV transformations are specified per-texture - see the 
# <tt>#AI_MATKEY_UVTRANSFORM<tt> material key for more information. 
# This step processes all textures with 
# transformed input UV coordinates and generates a new (pre-transformed) UV channel 
# which replaces the old channel. Most applications won't support UV 
# transformations, so you will probably want to specify this step.
#
# @note UV transformations are usually implemented in real-time apps by 
# transforming texture coordinates at vertex shader stage with a 3x3
# (homogenous) transformation matrix.
#
aiProcess_TransformUVCoords = 0x80000

## <hr>This step searches for duplicate meshes and replaces them
#  with references to the first mesh.
#
#  This step takes a while, so don't use it if speed is a concern.
#  Its main purpose is to workaround the fact that many export
#  file formats don't support instanced meshes, so exporters need to
#  duplicate meshes. This step removes the duplicates again. Please 
#  note that Assimp does not currently support per-node material
#  assignment to meshes, which means that identical meshes with
#  different materials are currently #not# joined, although this is 
#  planned for future versions.
#
aiProcess_FindInstances = 0x100000

## <hr>A postprocessing step to reduce the number of meshes.
#
#  This will, in fact, reduce the number of draw calls.
#
#  This is a very effective optimization and is recommended to be used
#  together with #aiProcess_OptimizeGraph, if possible. The flag is fully
#  compatible with both #aiProcess_SplitLargeMeshes and #aiProcess_SortByPType.
#
aiProcess_OptimizeMeshes  = 0x200000


## <hr>A postprocessing step to optimize the scene hierarchy. 
#
#  Nodes without animations, bones, lights or cameras assigned are 
#  collapsed and joined.
#
#  Node names can be lost during this step. If you use special 'tag nodes'
#  to pass additional information through your content pipeline, use the
#  <tt>#AI_CONFIG_PP_OG_EXCLUDE_LIST<tt> setting to specify a list of node 
#  names you want to be kept. Nodes matching one of the names in this list won't
#  be touched or modified.
#
#  Use this flag with caution. Most simple files will be collapsed to a 
#  single node, so complex hierarchies are usually completely lost. This is not
#  useful for editor environments, but probably a very effective
#  optimization if you just want to get the model data, convert it to your
#  own format, and render it as fast as possible. 
#
#  This flag is designed to be used with #aiProcess_OptimizeMeshes for best
#  results.
#
#  @note 'Crappy' scenes with thousands of extremely small meshes packed
#  in deeply nested nodes exist for almost all file formats.
#  #aiProcess_OptimizeMeshes in combination with #aiProcess_OptimizeGraph 
#  usually fixes them all and makes them renderable. 
#
aiProcess_OptimizeGraph  = 0x400000

## <hr>This step flips all UV coordinates along the y-axis and adjusts
# material settings and bitangents accordingly.
#
# <b>Output UV coordinate system:<b>
# @code
# 0y|0y ---------- 1x|0y 
# |                 |
# |                 |
# |                 |
# 0x|1y ---------- 1x|1y
# @endcode
#
# You'll probably want to consider this flag if you use Direct3D for
# rendering. The #aiProcess_ConvertToLeftHanded flag supersedes this
# setting and bundles all conversions typically required for D3D-based
# applications.
#
aiProcess_FlipUVs = 0x800000

## <hr>This step adjusts the output face winding order to be CW.
#
# The default face winding order is counter clockwise (CCW).
#
# <b>Output face order:<b>
# @code
#       x2
#           
#                         x0
#  x1
# @endcode
#
aiProcess_FlipWindingOrder  = 0x1000000

## <hr>This step splits meshes with many bones into sub-meshes so that each
# su-bmesh has fewer or as many bones as a given limit. 
#
aiProcess_SplitByBoneCount  = 0x2000000

## <hr>This step removes bones losslessly or according to some threshold.
#
#  In some cases (i.e. formats that require it) exporters are forced to
#  assign dummy bone weights to otherwise static meshes assigned to
#  animated meshes. Full, weight-based skinning is expensive while
#  animating nodes is extremely cheap, so this step is offered to clean up
#  the data in that regard. 
#                                                              
#  Use <tt>#AI_CONFIG_PP_DB_THRESHOLD<tt> to control this. 
#  Use <tt>#AI_CONFIG_PP_DB_ALL_OR_NONE<tt> if you want bones removed if and 
#  only if all bones within the scene qualify for removal.
#
aiProcess_Debone  = 0x4000000

aiProcess_GenEntityMeshes = 0x100000
aiProcess_OptimizeAnimations = 0x200000
aiProcess_FixTexturePaths = 0x200000
aiProcess_EmbedTextures  = 0x10000000,

## @def aiProcess_ConvertToLeftHanded
 #  @brief Shortcut flag for Direct3D-based applications. 
 #
 #  Supersedes the #aiProcess_MakeLeftHanded and #aiProcess_FlipUVs and
 #  #aiProcess_FlipWindingOrder flags.
 #  The output data matches Direct3D's conventions: left-handed geometry, upper-left
 #  origin for UV coordinates and finally clockwise face order, suitable for CCW culling.
 #
 #  @deprecated 
 #
aiProcess_ConvertToLeftHanded = ( \
    aiProcess_MakeLeftHanded     | \
    aiProcess_FlipUVs            | \
    aiProcess_FlipWindingOrder   | \
    0 ) 


## @def aiProcessPreset_TargetRealtimeUse_Fast
 #  @brief Default postprocess configuration optimizing the data for real-time rendering.
 #  
 #  Applications would want to use this preset to load models on end-user PCs,
 #  maybe for direct use in game.
 #
 # If you're using DirectX, don't forget to combine this value with
 # the #aiProcess_ConvertToLeftHanded step. If you don't support UV transformations
 # in your application apply the #aiProcess_TransformUVCoords step, too.
 #  @note Please take the time to read the docs for the steps enabled by this preset. 
 #  Some of them offer further configurable properties, while some of them might not be of
 #  use for you so it might be better to not specify them.
 #
aiProcessPreset_TargetRealtime_Fast = ( \
    aiProcess_CalcTangentSpace      |  \
    aiProcess_GenNormals            |  \
    aiProcess_JoinIdenticalVertices |  \
    aiProcess_Triangulate           |  \
    aiProcess_GenUVCoords           |  \
    aiProcess_SortByPType           |  \
    0 )

 ## @def aiProcessPreset_TargetRealtime_Quality
  #  @brief Default postprocess configuration optimizing the data for real-time rendering.
  #
  #  Unlike #aiProcessPreset_TargetRealtime_Fast, this configuration
  #  performs some extra optimizations to improve rendering speed and
  #  to minimize memory usage. It could be a good choice for a level editor
  #  environment where import speed is not so important.
  #  
  #  If you're using DirectX, don't forget to combine this value with
  #  the #aiProcess_ConvertToLeftHanded step. If you don't support UV transformations
  #  in your application apply the #aiProcess_TransformUVCoords step, too.
  #  @note Please take the time to read the docs for the steps enabled by this preset. 
  #  Some of them offer further configurable properties, while some of them might not be
  #  of use for you so it might be better to not specify them.
  #
aiProcessPreset_TargetRealtime_Quality = ( \
    aiProcess_CalcTangentSpace              |  \
    aiProcess_GenSmoothNormals              |  \
    aiProcess_JoinIdenticalVertices         |  \
    aiProcess_ImproveCacheLocality          |  \
    aiProcess_LimitBoneWeights              |  \
    aiProcess_RemoveRedundantMaterials      |  \
    aiProcess_SplitLargeMeshes              |  \
    aiProcess_Triangulate                   |  \
    aiProcess_GenUVCoords                   |  \
    aiProcess_SortByPType                   |  \
    aiProcess_FindDegenerates               |  \
    aiProcess_FindInvalidData               |  \
    0 )

 ## @def aiProcessPreset_TargetRealtime_MaxQuality
  #  @brief Default postprocess configuration optimizing the data for real-time rendering.
  #
  #  This preset enables almost every optimization step to achieve perfectly
  #  optimized data. It's your choice for level editor environments where import speed 
  #  is not important.
  #  
  #  If you're using DirectX, don't forget to combine this value with
  #  the #aiProcess_ConvertToLeftHanded step. If you don't support UV transformations
  #  in your application, apply the #aiProcess_TransformUVCoords step, too.
  #  @note Please take the time to read the docs for the steps enabled by this preset. 
  #  Some of them offer further configurable properties, while some of them might not be
  #  of use for you so it might be better to not specify them.
  #
aiProcessPreset_TargetRealtime_MaxQuality = ( \
    aiProcessPreset_TargetRealtime_Quality   |  \
    aiProcess_FindInstances                  |  \
    aiProcess_ValidateDataStructure          |  \
    aiProcess_OptimizeMeshes                 |  \
    0 )





import os, inspect
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
print ("current_dir=" + currentdir)

import pybullet as p
import pybullet_data



import time
cid = p.connect(p.SHARED_MEMORY)
if (cid < 0):
    p.connect(p.GUI)

p.resetSimulation()
p.setGravity(0, 0, -10)
p.loadSDF(os.path.join(pybullet_data.getDataPath(),"stadium.sdf"))
useRealTimeSim = 1

# for video recording (works best on Mac and Linux, not well on Windows)
# p.startStateLogging(p.STATE_LOGGING_VIDEO_MP4, "racecar.mp4")
p.setRealTimeSimulation(useRealTimeSim)  # either this

object_ids = p.loadURDF(os.path.join(currentdir, 'models', 'husky.urdf'), flags=p.URDF_USE_SELF_COLLISION+p.URDF_USE_SELF_COLLISION_EXCLUDE_ALL_PARENTS)

print(object_ids)
while (True):
    time.sleep(0.01)


from gibson.core.physics.scene_abstract import Scene
import pybullet as p
import os
import pybullet_data

class StadiumScene(Scene):
    zero_at_running_strip_start_line = True   # if False, center of coordinates (0,0,0) will be at the middle of the stadium
    stadium_halflen   = 105*0.25    # FOOBALL_FIELD_HALFLEN
    stadium_halfwidth = 50*0.25  # FOOBALL_FIELD_HALFWID

    def __init__(self, robot, gravity, timestep, frame_skip, env=None):
        Scene.__init__(self, gravity, timestep, frame_skip, env)

        filename = os.path.join(pybullet_data.getDataPath(), "stadium_no_collision.sdf")
        self.stadium = p.loadSDF(filename)
        planeName = os.path.join(pybullet_data.getDataPath(), "mjcf/ground_plane.xml")
        self.ground_plane_mjcf = p.loadMJCF(planeName)
        for i in self.ground_plane_mjcf:
            pos, orn = p.getBasePositionAndOrientation(i)
            p.resetBasePositionAndOrientation(i, [pos[0], pos[1], pos[2] - 0.005], orn)

        for i in self.ground_plane_mjcf:
            p.changeVisualShape(i, -1, rgbaColor=[1, 1, 1, 0.5])

        self.scene_obj_list = self.stadium


    def episode_restart(self):
        Scene.episode_restart(self)   # contains cpp_world.clean_everything()


class SinglePlayerStadiumScene(StadiumScene):
    "This scene created by environment, to work in a way as if there was no concept of scene visible to user."
    multiplayer = False

class MultiplayerStadiumScene(StadiumScene):
    multiplayer = True
    players_count = 3
    def actor_introduce(self, robot):
        StadiumScene.actor_introduce(self, robot)
        i = robot.player_n - 1  # 0 1 2 => -1 0 +1
        robot.move_robot(0, i, 0)

if __name__ == "__main__":

    cid = p.connect(p.SHARED_MEMORY)
    if (cid < 0):
        p.connect(p.GUI)

    scene = StadiumScene(gravity=9.8, timestep=0.0165, frame_skip=1)
    scene.episode_restart()
    while True:
        pass


'''
Customized Minitaur Environment for Cambria
Author: Zhiyang He, Stanford University
Original: Pybullet

Note that in the original pybullet environment, major difference exist in simulation
accuracy. 
Original:
    Solver iterations: 300 (Major difference)
    Time step: 1/100.0
    Action repeat: 1
Original Accurate:
    Solver iterations: 60
    Time step: 1/500.0
    Action repeat: 5
Current:
    Solver iterations: 5
    Time step: 1/88.0
    Action repeat: 4
'''


from gibson.envs.env_modalities import CameraRobotEnv, BaseRobotEnv
from gibson.envs.env_bases import *
from gibson.core.physics.drivers.minitaur import Minitaur
import os, inspect
import math
import time
import gym
from gym import spaces
from gym.utils import seeding
import gibson
import numpy as np
import pybullet

MINITAUR_TIMESTEP  = 1.0/(4 * 22)
MINITAUR_FRAMESKIP = 4

ACTION_EPS = 0.01


tracking_camera = {
    'yaw': 40,
    'z_offset': 0.3,
    'distance': 1,
    'pitch': -0
}


class MinitaurNavigateEnv(CameraRobotEnv):
    """The gym environment for the minitaur.

    It simulates the locomotion of a minitaur, a quadruped robot. The state space
    include the angles, velocities and torques for all the motors and the action
    space is the desired motor angle for each motor. The reward function is based
    on how far the minitaur walks in 1000 steps and penalizes the energy
    expenditure.
    """
    distance_weight = 1.0
    energy_weight = 0.005
    shake_weight = 0.0
    drift_weight = 0.0
    distance_limit = float("inf")
    observation_noise_stdev = 0.0
    action_bound = 1
    env_randomizer = None
    hard_reset = False

    leg_model_enabled = True
    num_bullet_solver_iterations = 300    
    pd_control_enabled = True
    accurate_motor_model_enabled = True
    NUM_SUBSTEPS = 5     # PD control needs smaller time step for stability.


    def __init__(self, config, gpu_idx=0):
        """Initialize the minitaur gym environment.
        Args:
            distance_weight: The weight of the distance term in the reward.
            energy_weight: The weight of the energy term in the reward.
            shake_weight: The weight of the vertical shakiness term in the reward.
            drift_weight: The weight of the sideways drift term in the reward.
            distance_limit: The maximum distance to terminate the episode.
            observation_noise_stdev: The standard deviation of observation noise.
            leg_model_enabled: Whether to use a leg motor to reparameterize the action
                space.
            hard_reset: Whether to wipe the simulation and load everything when reset
                is called. If set to false, reset just place the minitaur back to start
                position and set its pose to initial configuration.
            env_randomizer: An EnvRandomizer to randomize the physical properties
                during reset().
        """
    
        self.config = self.parse_config(config)
        assert(self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")

        CameraRobotEnv.__init__(self, self.config, gpu_idx,
                                scene_type="building",
                                tracking_camera=tracking_camera)

        self.robot_introduce(Minitaur(self.config, env=self, 
                                      pd_control_enabled=self.pd_control_enabled,
                                      accurate_motor_model_enabled=self.accurate_motor_model_enabled))
        self.scene_introduce()
        self.gui = self.config["mode"] == "gui"
        self.total_reward = 0
        self.total_frame = 0

        self.action_repeat = 1
        ## Important: PD controller needs more accuracy
        '''if self.pd_control_enabled or self.accurate_motor_model_enabled:
            self.time_step = self.config["speed"]["timestep"]
            self.time_step /= self.NUM_SUBSTEPS
            self.num_bullet_solver_iterations /= self.NUM_SUBSTEPS
            self.action_repeat *= self.NUM_SUBSTEPS
            pybullet.setPhysicsEngineParameter(physicsClientId=self.physicsClientId,
              numSolverIterations=int(self.num_bullet_solver_iterations))
            pybullet.setTimeStep(self.time_step, physicsClientId=self.physicsClientId)
        '''
        pybullet.setPhysicsEngineParameter(physicsClientId=self.physicsClientId,
              numSolverIterations=int(self.num_bullet_solver_iterations))
        self._observation = []
        self._last_base_position = [0, 0, 0]
        self._action_bound = self.action_bound
        
        self._env_randomizer = self.env_randomizer        
        if self._env_randomizer is not None:
            self._env_randomizer.randomize_env(self)

        self._objectives = []        
        self.viewer = None
        self.Amax = [0] * 8

    def set_env_randomizer(self, env_randomizer):
        self._env_randomizer = env_randomizer

    def configure(self, args):
        self._args = args

    #def _reset(self):
        #if self._env_randomizer is not None:
        #    self._env_randomizer.randomize_env(self)

        #self._last_base_position = [0, 0, 0]
        #self._objectives = []
        
        #if not self._torque_control_enabled:
        #  for _ in range(1 / self.timestep):
        #    if self._pd_control_enabled or self._accurate_motor_model_enabled:
        #    self.robot.ApplyAction([math.pi / 2] * 8)
        #    pybullet.stepSimulation()
        #return self._noisy_observation()


    def _transform_action_to_motor_command(self, action):
        if self.leg_model_enabled:
            #for i, action_component in enumerate(action):
            #    if not (-self._action_bound - ACTION_EPS <= action_component <= self._action_bound + ACTION_EPS):
            #        raise ValueError("{}th action {} out of bounds.".format(i, action_component))
            action = self.robot.ConvertFromLegModel(action)
        return action
    
    def _step(self, action):
        """Step forward the simulation, given the action.

        Args:
          action: A list of desired motor angles for eight motors.

        Returns:
          observations: The angles, velocities and torques of all motors.
          reward: The reward for the current state-action pair.
          done: Whether the episode has ended.
          info: A dictionary that stores diagnostic information.

        Raises:
          ValueError: The action dimension is not the same as the number of motors.
          ValueError: The magnitude of actions is out of bounds.
        """
        #print("Env apply raw action", action)
        action = self._transform_action_to_motor_command(action)
        #print("Env apply action", action)
    
        #for _ in range(self._action_repeat):
        #  self.robot.ApplyAction(action)
        #  pybullet.stepSimulation()
        for i in range(len(self.Amax)):
            if action[i] > self.Amax[i]:
                self.Amax[i] = action[i]
        #print("Action max", self.Amax)
        for _ in range(self.action_repeat):
            state = CameraRobotEnv._step(self, action)
        return state

    step  = _step

    def calc_rewards_and_done(self, action, state):
        ## TODO (hzyjerry): make use of action, state
        done = self._termination(state)
        rewards = self._rewards(a)
        #return reward, False
        return rewards, done


    def get_minitaur_motor_angles(self):
        """Get the minitaur's motor angles.

        Returns:
          A numpy array of motor angles.
        """
        return self.robot.GetMotorAngles()
    
    def get_minitaur_motor_velocities(self):
        """Get the minitaur's motor velocities.

        Returns:
          A numpy array of motor velocities.
        """
        return self.robot.GetMotorVelocities()

    def get_minitaur_motor_torques(self):
        """Get the minitaur's motor torques.

        Returns:
          A numpy array of motor torques.
        """
        return self.robot.GetMotorTorques()

    def get_minitaur_base_orientation(self):
        """Get the minitaur's base orientation, represented by a quaternion.

        Returns:
          A numpy array of minitaur's orientation.
        """
        return self.robot.GetBaseOrientation()

    def is_fallen(self):
        """Decide whether the minitaur has fallen.

        If the up directions between the base and the world is larger (the dot
        product is smaller than 0.85) or the base is very low on the ground
        (the height is smaller than 0.13 meter), the minitaur is considered fallen.

        Returns:
          Boolean value that indicates whether the minitaur has fallen.
        """
        orientation = self.robot.GetBaseOrientation()
        rot_mat = pybullet.getMatrixFromQuaternion(orientation)
        local_up = rot_mat[6:]
        pos = self.robot.GetBasePosition()
        #return (np.dot(np.asarray([0, 0, 1]), np.asarray(local_up)) < 0.85 or
        #        pos[2] < 0.13)
        return False

    def _termination(self, state=None, debugmode=False):
        position = self.robot.GetBasePosition()
        distance = math.sqrt(position[0]**2 + position[1]**2)
        #return self.is_fallen() or distance > self.distance_limit
        return False

    def _rewards(self, action=None, debugmode=False):
        a = action
        current_base_position = self.robot.GetBasePosition()
        forward_reward = current_base_position[0] - self._last_base_position[0]
        drift_reward = -abs(current_base_position[1] - self._last_base_position[1])
        shake_reward = -abs(current_base_position[2] - self._last_base_position[2])
        self._last_base_position = current_base_position
        energy_reward = np.abs(
            np.dot(self.robot.GetMotorTorques(),
                   self.robot.GetMotorVelocities())) * self.timestep
        reward = (
            self.distance_weight * forward_reward -
            self.energy_weight * energy_reward + self.drift_weight * drift_reward
            + self.shake_weight * shake_reward)
        self._objectives.append(
            [forward_reward, energy_reward, drift_reward, shake_reward])
        return [reward, ]

    def get_objectives(self):
        return self._objectives

    def _get_observation(self):
        self._observation = self.robot.GetObservation()
        return self._observation

    def _noisy_observation(self):
        self._get_observation()
        observation = np.array(self._observation)
        if self.observation_noise_stdev > 0:
          observation += (np.random.normal(
              scale=self.observation_noise_stdev, size=observation.shape) *
                          self.robot.GetObservationUpperBound())
        return observation


    #==================== Environemnt Randomizer ====================
    ## (hzyjerry) TODO: still under construction, not ready to use

    def randomize_env(self, env):
        self._randomize_minitaur(env.minitaur)

    def _randomize_minitaur(self, minitaur):
        """Randomize various physical properties of minitaur.

        It randomizes the mass/inertia of the base, mass/inertia of the legs,
        friction coefficient of the feet, the battery voltage and the motor damping
        at each reset() of the environment.

        Args:
          minitaur: the Minitaur instance in minitaur_gym_env environment.
        """
        base_mass = minitaur.GetBaseMassFromURDF()
        randomized_base_mass = random.uniform(
            base_mass * (1.0 + self._minitaur_base_mass_err_range[0]),
            base_mass * (1.0 + self._minitaur_base_mass_err_range[1]))
        minitaur.SetBaseMass(randomized_base_mass)

        leg_masses = minitaur.GetLegMassesFromURDF()
        leg_masses_lower_bound = np.array(leg_masses) * (
            1.0 + self._minitaur_leg_mass_err_range[0])
        leg_masses_upper_bound = np.array(leg_masses) * (
            1.0 + self._minitaur_leg_mass_err_range[1])
        randomized_leg_masses = [
            np.random.uniform(leg_masses_lower_bound[i], leg_masses_upper_bound[i])
            for i in range(len(leg_masses))
        ]
        minitaur.SetLegMasses(randomized_leg_masses)

        randomized_battery_voltage = random.uniform(BATTERY_VOLTAGE_RANGE[0],
                                                    BATTERY_VOLTAGE_RANGE[1])
        minitaur.SetBatteryVoltage(randomized_battery_voltage)

        randomized_motor_damping = random.uniform(MOTOR_VISCOUS_DAMPING_RANGE[0],
                                                  MOTOR_VISCOUS_DAMPING_RANGE[1])
        minitaur.SetMotorViscousDamping(randomized_motor_damping)

        randomized_foot_friction = random.uniform(MINITAUR_LEG_FRICTION[0],
                                                  MINITAUR_LEG_FRICTION[1])
        minitaur.SetFootFriction(randomized_foot_friction)



from gibson.envs.env_modalities import CameraRobotEnv, BaseRobotEnv
from gibson.envs.env_bases import *
from gibson.core.physics.robot_locomotors import Quadrotor
from transforms3d import quaternions
import os
import numpy as np
import sys
import pybullet as p
from gibson.core.physics.scene_stadium import SinglePlayerStadiumScene
import pybullet_data
import cv2

CALC_OBSTACLE_PENALTY = 1

tracking_camera = {
    'yaw': 20,
    'z_offset': 0.5,
    'distance': 1,
    'pitch': -20
}

tracking_camera_top = {
    'yaw': 20,  # demo: living room, stairs
    'z_offset': 0.5,
    'distance': 1,
    'pitch': -20
}

class DroneNavigateEnv(CameraRobotEnv):
    """Specfy navigation reward
    """
    def __init__(self, config, gpu_idx=0):
        self.config = self.parse_config(config)
        print(self.config["envname"])
        assert(self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")
        CameraRobotEnv.__init__(self, self.config, gpu_idx,
                                scene_type="building",
                                tracking_camera=tracking_camera)

        self.robot_introduce(Quadrotor(self.config, env=self))
        self.scene_introduce()
        self.gui = self.config["mode"] == "gui"
        self.total_reward = 0
        self.total_frame = 0
        assert(self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")


    def add_text(self, img):
        font = cv2.FONT_HERSHEY_SIMPLEX
        x,y,z = self.robot.body_xyz
        r,p,ya = self.robot.body_rpy
        cv2.putText(img, 'x:{0:.4f} y:{1:.4f} z:{2:.4f}'.format(x,y,z), (10, 20), font, 0.5, (255, 255, 255), 1, cv2.LINE_AA)
        cv2.putText(img, 'ro:{0:.4f} pth:{1:.4f} ya:{2:.4f}'.format(r,p,ya), (10, 40), font, 0.5, (255, 255, 255), 1, cv2.LINE_AA)
        cv2.putText(img, 'potential:{0:.4f}'.format(self.potential), (10, 60), font, 0.5, (255, 255, 255), 1, cv2.LINE_AA)
        cv2.putText(img, 'fps:{0:.4f}'.format(self.fps), (10, 80), font, 0.5, (255, 255, 255), 1, cv2.LINE_AA)
        return img

    def _rewards(self, action=None, debugmode=False):
        a = action
        potential_old = self.potential
        self.potential = self.robot.calc_potential()
        progress = float(self.potential - potential_old)

        feet_collision_cost = 0.0
        for i, f in enumerate(
                self.robot.feet):  # TODO: Maybe calculating feet contacts could be done within the robot code
            # print(f.contact_list())
            contact_ids = set((x[2], x[4]) for x in f.contact_list())
            # print("CONTACT OF '%d' WITH %d" % (contact_ids, ",".join(contact_names)) )
            if (self.ground_ids & contact_ids):
                # see Issue 63: https://github.com/openai/roboschool/issues/63
                # feet_collision_cost += self.foot_collision_cost
                self.robot.feet_contact[i] = 1.0
            else:
                self.robot.feet_contact[i] = 0.0
        # print(self.robot.feet_contact)

        #electricity_cost  = self.electricity_cost  * float(np.abs(a*self.robot.joint_speeds).mean())  # let's assume we 
        electricity_cost  = self.stall_torque_cost * float(np.square(a).mean())


        debugmode = 0
        wall_contact = [pt for pt in self.robot.parts['base_link'].contact_list() if pt[6][2] > 0.15]
        wall_collision_cost = self.wall_collision_cost * len(wall_contact)

        joints_at_limit_cost = float(self.joints_at_limit_cost * self.robot.joints_at_limit)
        close_to_goal = 0
        if self.robot.dist_to_target() < 2:
            close_to_goal = 0.5

        obstacle_penalty = 0

        debugmode = 0

        debugmode = 0
        if (debugmode):
            print("Wall contact points", len(wall_contact))
            print("Collision cost", wall_collision_cost)
            print("electricity_cost", electricity_cost)
            print("close to goal", close_to_goal)
            #print("progress")
            #print(progress)
            #print("electricity_cost")
            #print(electricity_cost)
            #print("joints_at_limit_cost")
            #print(joints_at_limit_cost)
            #print("feet_collision_cost")
            #print(feet_collision_cost)

        rewards = [
            #alive,
            progress,
            #wall_collision_cost,
            close_to_goal,
            obstacle_penalty
            #electricity_cost,
            #joints_at_limit_cost,
            #feet_collision_cost
        ]
        return rewards

    def _termination(self, debugmode=False):

        done = self.nframe > 250 or self.robot.get_position()[2] < 0
        #if done:
        #    print("Episode reset")
        return done

    def  _reset(self):
        self.total_frame = 0
        self.total_reward = 0

        obs = CameraRobotEnv._reset(self)
        return obs



from gibson.envs.env_modalities import CameraRobotEnv, BaseRobotEnv
from gibson.envs.env_bases import *
from gibson.core.physics.robot_locomotors import Humanoid, Ant
from transforms3d import quaternions
import os
import numpy as np
import sys
import pybullet as p
from gibson.core.physics.scene_stadium import SinglePlayerStadiumScene
import pybullet_data


tracking_camera = {
    'yaw': 20,  # demo: living room, stairs
    #'yaw'; 30,   # demo: kitchen
    'z_offset': 0.5,
    'distance': 1.2,
    'pitch': -20
    # 'pitch': -24  # demo: stairs
}


class HumanoidNavigateEnv(CameraRobotEnv):
    """Specfy navigation reward
    """
    def __init__(self, config, gpu_idx=0):
        self.config = self.parse_config(config)
        print(self.config["envname"])
        assert(self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")
        CameraRobotEnv.__init__(self, self.config, gpu_idx,
                                scene_type="building",
                                tracking_camera=tracking_camera)

        self.robot_introduce(Humanoid(self.config, env=self))
        self.scene_introduce()

        self.gui = self.config["mode"] == "gui"
        self.total_reward = 0
        self.total_frame = 0

    def _rewards(self, action=None, debugmode=False):
        a = action
        potential_old = self.potential
        self.potential = self.robot.calc_potential()
        progress = float(self.potential - potential_old)

        feet_collision_cost = 0.0
        for i,f in enumerate(self.robot.feet): # TODO: Maybe calculating feet contacts could be done within the robot code
            #print(f.contact_list())
            contact_ids = set((x[2], x[4]) for x in f.contact_list())
            #print("CONTACT OF '%d' WITH %d" % (contact_ids, ",".join(contact_names)) )
            if (self.ground_ids & contact_ids):
                #see Issue 63: https://github.com/openai/roboschool/issues/63
                #feet_collision_cost += self.foot_collision_cost
                self.robot.feet_contact[i] = 1.0
            else:
                self.robot.feet_contact[i] = 0.0
            #print(self.robot.feet_contact)
        electricity_cost  = self.electricity_cost  * float(np.abs(a*self.robot.joint_speeds).mean())  # let's assume we have DC motor with controller, and reverse current braking
        electricity_cost += self.stall_torque_cost * float(np.square(a).mean())
        joints_at_limit_cost = float(self.joints_at_limit_cost * self.robot.joints_at_limit)
        
        if(debugmode):
            print("progress")
            print(progress)
            print("electricity_cost")
            print(electricity_cost)
            print("joints_at_limit_cost")
            print(joints_at_limit_cost)
            print("feet_collision_cost")
            print(feet_collision_cost)

        rewards =[
            #alive,
            progress,
            electricity_cost,
            joints_at_limit_cost,
            feet_collision_cost
            ]
        return rewards

    def _termination(self, debugmode=False):
        height = self.robot.get_position()[2]
        pitch = self.robot.get_rpy()[1]
        alive = float(self.robot.alive_bonus(height, pitch))
        done = alive < 0
        if(debugmode):
            print("alive=")
            print(alive)
        return done

    def _flag_reposition(self):
        walk_target_x, walk_target_y, _ = self.robot.get_target_position()

        self.flag = None
        if self.gui and not self.config["display_ui"]:
            self.visual_flagId = p.createVisualShape(p.GEOM_MESH, fileName=os.path.join(pybullet_data.getDataPath(), 'cube.obj'), meshScale=[0.5, 0.5, 0.5], rgbaColor=[1, 0, 0, 0])
            self.last_flagId = p.createMultiBody(baseVisualShapeIndex=self.visual_flagId, baseCollisionShapeIndex=-1, basePosition=[walk_target_x, walk_target_y, 0.5])
        
    def _reset(self):
        self.total_frame = 0
        self.total_reward = 0
        obs = CameraRobotEnv._reset(self)
        self._flag_reposition()
        return obs


class HumanoidGibsonFlagRunEnv(CameraRobotEnv):
    """Specfy flagrun reward
    """

    def __init__(self, config, gpu_idx=0):
        self.config = self.parse_config(config)
        print(self.config["envname"])
        assert(self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")
        CameraRobotEnv.__init__(self, self.config, gpu_idx,
                                scene_type="building",
                                tracking_camera=tracking_camera)

        self.robot_introduce(Humanoid(self.config, env=self))
        self.scene_introduce()

        self.gui = self.config["mode"] == "gui"
        self.total_reward = 0
        self.total_frame = 0

        self.flag_timeout = 1
        self.visualid = -1
        self.lastid = None
        if self.gui:
            self.visualid = p.createVisualShape(p.GEOM_MESH,
                                                fileName=os.path.join(pybullet_data.getDataPath(), 'cube.obj'),
                                                meshScale=[0.2, 0.2, 0.2], rgbaColor=[1, 0, 0, 0.7])
        self.colisionid = p.createCollisionShape(p.GEOM_MESH,
                                                 fileName=os.path.join(pybullet_data.getDataPath(), 'cube.obj'),
                                                 meshScale=[0.2, 0.5, 0.2])
        assert (self.config["envname"] == self.__class__.__name__ or self.config["envname"] == "TestEnv")

    def _reset(self):
        obs = CameraRobotEnv._reset(self)
        return obs

    def _flag_reposition(self):
        # self.walk_target_x = self.np_random.uniform(low=-self.scene.stadium_halflen,
        #                                            high=+self.scene.stadium_halflen)
        # self.walk_target_y = self.np_random.uniform(low=-self.scene.stadium_halfwidth,
        #                                            high=+self.scene.stadium_halfwidth)
        force_x = self.np_random.uniform(-300, 300)
        force_y = self.np_random.uniform(-300, 300)

        more_compact = 0.5  # set to 1.0 whole football field
        # self.walk_target_x *= more_compact
        # self.walk_target_y *= more_compact

        startx, starty, _ = self.robot.body_xyz

        self.flag = None
        # self.flag = self.scene.cpp_world.debug_sphere(self.walk_target_x, self.walk_target_y, 0.2, 0.2, 0xFF8080)
        self.flag_timeout = 3000 / self.scene.frame_skip
        # print('targetxy', self.flagid, self.walk_target_x, self.walk_target_y, p.getBasePositionAndOrientation(self.flagid))
        # p.resetBasePositionAndOrientation(self.flagid, posObj = [self.walk_target_x, self.walk_target_y, 0.5], ornObj = [0,0,0,0])
        if self.lastid:
            p.removeBody(self.lastid)

        self.lastid = p.createMultiBody(baseMass=1, baseVisualShapeIndex=self.visualid,
                                        baseCollisionShapeIndex=self.colisionid, basePosition=[startx, starty, 0.5])
        p.applyExternalForce(self.lastid, -1, [force_x, force_y, 50], [0, 0, 0], p.LINK_FRAME)

        ball_xyz, _ = p.getBasePositionAndOrientation(self.lastid)

        self.robot.walk_target_x = ball_xyz[0]
        self.robot.walk_target_y = ball_xyz[1]

    def _rewards(self, action=None, debugmode=False):
        a = action
        potential_old = self.potential
        self.potential = self.robot.calc_potential()
        progress = float(self.potential - potential_old)

        feet_collision_cost = 0.0
        for i, f in enumerate(
                self.robot.feet):  # TODO: Maybe calculating feet contacts could be done within the robot code
            # print(f.contact_list())
            contact_ids = set((x[2], x[4]) for x in f.contact_list())
            # print("CONTACT OF '%d' WITH %d" % (contact_ids, ",".join(contact_names)) )
            if (self.ground_ids & contact_ids):
                # see Issue 63: https://github.com/openai/roboschool/issues/63
                # feet_collision_cost += self.foot_collision_cost
                self.robot.feet_contact[i] = 1.0
            else:
                self.robot.feet_contact[i] = 0.0
                # print(self.robot.feet_contact)
        electricity_cost = self.electricity_cost * float(np.abs(
            a * self.robot.joint_speeds).mean())  # let's assume we have DC motor with controller, and reverse current braking
        electricity_cost += self.stall_torque_cost * float(np.square(a).mean())
        joints_at_limit_cost = float(self.joints_at_limit_cost * self.robot.joints_at_limit)

        if (debugmode):
            print("progress")
            print(progress)
            print("electricity_cost")
            print(electricity_cost)
            print("joints_at_limit_cost")
            print(joints_at_limit_cost)
            print("feet_collision_cost")
            print(feet_collision_cost)

        rewards = [
            # alive,
            progress,
            electricity_cost,
            joints_at_limit_cost,
            feet_collision_cost
        ]
        return rewards

    def _termination(self, debugmode=False):
        height = self.robot.get_position()[2]
        alive = float(self.robot.alive_bonus(height, self.robot.get_rpy()[
            1]))
        done = alive < 0
        if (debugmode):
            print("alive=")
            print(alive)
        return done

    def _step(self, a):
        state, reward, done, meta = CameraRobotEnv._step(self, a)
        if self.flag_timeout <= 0:
            self._flag_reposition()
        self.flag_timeout -= 1

        return state, reward, done, meta

    ## openai-gym v0.10.5 compatibility
    step  = _step



from PIL import Image
import numpy as np
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt

#opengl_path  = "/home/jerry/Pictures/point_0_view_2_domain_fixatedpose.png"

opengl_path  = "/home/jerry/Pictures/point_0_view_2_domain_fixatedpose_mist.png"
#blender_path = "/home/jerry/Desktop/Data/1CzjpjNF8qk/depth/point_0_view_2_domain_depth.png"
blender_path = "/home/jerry/Desktop/Data/1CzjpjNF8qk/mist/point_0_view_2_domain_mist.png"

outline_path  = "/home/jerry/Pictures/point_0_view_2_domain_outline.png"

opengl_viz  = "/home/jerry/Pictures/point_0_view_2_domain_viz.png"

opengl_img  = Image.open(opengl_path)
blender_img = Image.open(blender_path)


opengl_arr  = np.asarray(opengl_img)
blender_arr = np.asarray(blender_img)

## Opengl:  opengl_arr[:, :, 0]
## Blender: blender_arr

#opengl_arr  = opengl_arr[:, :, 0].reshape((1, -1))[0]  ## fpa version


# quick fix for visualization
## TODO: cpp png saving upside down
#opengl_arr 
opengl_arr = opengl_arr[::-1][:]
#print(opengl_arr.shape, blender_arr.shape)


outline_arr = opengl_arr.copy()

for row in range(opengl_arr.shape[0]):
    for col in range(opengl_arr.shape[1]):
        if np.abs(opengl_arr[row][col] - blender_arr[row][col]) > 3:
            print(opengl_arr[row][col], blender_arr[row][col])
            outline_arr[row][col] = 65535


im = Image.new('L', (512, 512))
im.putdata(outline_arr.flatten().tolist())
im.save(outline_path)

viz_arr = opengl_arr.copy()
viz_arr = viz_arr * 128.0 / 65535
viz_arr = np.power(viz_arr, 5)
print(viz_arr)


im = Image.new('L', (512, 512))
im.putdata(viz_arr.flatten().tolist())
im.save(opengl_viz)

opengl_arr  = opengl_arr.reshape((1, -1))[0]  ## png version
blender_arr = blender_arr.reshape((1, -1))[0]
print("before clamping, max blender: ", max(blender_arr)) 
print(opengl_arr)
print(np.min(opengl_arr), np.max(opengl_arr), len(opengl_arr))
print(np.min(blender_arr),np.max(blender_arr), len(blender_arr))

diff_count = np.sum((opengl_arr != blender_arr))
diff_sqsum = np.sum(np.square(opengl_arr - blender_arr))

total_count = len(opengl_arr)
total_sqsum = np.sum(np.square(opengl_arr))
print('How many different', diff_count, float(diff_count) / total_count)
print('Total square diff', diff_sqsum, float(diff_sqsum) / total_sqsum)

blender_arr = blender_arr[blender_arr < 65535]

plt.subplot(2, 1, 1)
n, bins, patches = plt.hist(opengl_arr, 50, normed=1, label='opengl', alpha=0.75)
plt.legend(loc='upper right')
plt.subplot(2, 1, 2)
n, bins, patches = plt.hist(blender_arr, 50, normed=1, label='blender', alpha=0.75)
plt.legend(loc='upper right')
plt.show()




from flask import Flask, render_template, Response
import sys
import pickle
import socket
import zmq

app = Flask(__name__)

port = "5552"
if len(sys.argv) > 1:
    port =  sys.argv[1]

if len(sys.argv) > 2:
    port_web =  sys.argv[2]
    port_web = int(port_web)
else:
    port_web = 5001
# Socket to talk to server
context = zmq.Context()
socket = context.socket(zmq.SUB)

print("Collecting updates from server...")
socket.connect ("tcp://localhost:%s" % port)


topicfilter = b"ui"
socket.setsockopt(zmq.SUBSCRIBE, topicfilter)

@app.route('/')
def index():
    return render_template('index.html')

def gen():
    while True:
        string = socket.recv()
        
        data = string[2:]
        #from IPython import embed; embed()
        frame = pickle.loads(data)[-1]
        
        frame = frame.tobytes()
        #print(frame.shape)
        yield (b'--frame\r\n'
               b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n\r\n')

@app.route('/video_feed')
def video_feed():
    return Response(gen(),
                    mimetype='multipart/x-mixed-replace; boundary=frame')

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=port_web, debug=False)



# Copyright 2013 Google Inc. All Rights Reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Shared utilities for writing scripts for Google Test/Mock."""

__author__ = 'wan@google.com (Zhanyong Wan)'


import os
import re


# Matches the line from 'svn info .' output that describes what SVN
# path the current local directory corresponds to.  For example, in
# a googletest SVN workspace's trunk/test directory, the output will be:
#
# URL: https://googletest.googlecode.com/svn/trunk/test
_SVN_INFO_URL_RE = re.compile(r'^URL: https://(\w+)\.googlecode\.com/svn(.*)')


def GetCommandOutput(command):
  """Runs the shell command and returns its stdout as a list of lines."""

  f = os.popen(command, 'r')
  lines = [line.strip() for line in f.readlines()]
  f.close()
  return lines


def GetSvnInfo():
  """Returns the project name and the current SVN workspace's root path."""

  for line in GetCommandOutput('svn info .'):
    m = _SVN_INFO_URL_RE.match(line)
    if m:
      project = m.group(1)  # googletest or googlemock
      rel_path = m.group(2)
      root = os.path.realpath(rel_path.count('/') * '../')
      return project, root

  return None, None


def GetSvnTrunk():
  """Returns the current SVN workspace's trunk root path."""

  _, root = GetSvnInfo()
  return root + '/trunk' if root else None


def IsInGTestSvn():
  project, _ = GetSvnInfo()
  return project == 'googletest'


def IsInGMockSvn():
  project, _ = GetSvnInfo()
  return project == 'googlemock'



import gym
#import pygame
import sys
import time
import matplotlib
import time
import pygame
import pybullet as p
from gibson.core.render.profiler import Profiler
'''
try:
    matplotlib.use('GTK3Agg')
    import matplotlib.pyplot as plt
except Exception:
    pass
'''

#import pyglet.window as pw

from collections import deque
#from pygame.locals import HWSURFACE, DOUBLEBUF, RESIZABLE, VIDEORESIZE
from threading import Thread

def display_arr(screen, arr, video_size, transpose):
    arr_min, arr_max = arr.min(), arr.max()
    arr = 255.0 * (arr - arr_min) / (arr_max - arr_min)
    pyg_img = pygame.surfarray.make_surface(arr.swapaxes(0, 1) if transpose else arr)
    pyg_img = pygame.transform.scale(pyg_img, video_size)
    screen.blit(pyg_img, (0,0))

def play(env, transpose=True, zoom=None, callback=None, keys_to_action=None):
    """Allows one to play the game using keyboard.

    To simply play the game use:

        play(gym.make("Pong-v3"))
        play(env)

    Above code works also if env is wrapped, so it's particularly useful in
    verifying that the frame-level preprocessing does not render the game
    unplayable.

    If you wish to plot real time statistics as you play, you can use
    gym.utils.play.PlayPlot. Here's a sample code for plotting the reward
    for last 5 second of gameplay.

        def callback(obs_t, obs_tp1, rew, done, info):
            return [rew,]
        env_plotter = EnvPlotter(callback, 30 * 5, ["reward"])

        env = gym.make("Pong-v3")
        play(env, callback=env_plotter.callback)


    Arguments
    ---------
    env: gym.Env
        Environment to use for playing.
    transpose: bool
        If True the output of observation is transposed.
        Defaults to true.
    zoom: float
        Make screen edge this many times bigger
    callback: lambda or None
        Callback if a callback is provided it will be executed after
        every step. It takes the following input:
            obs_t: observation before performing action
            obs_tp1: observation after performing action
            action: action that was executed
            rew: reward that was received
            done: whether the environemnt is done or not
            info: debug info
    keys_to_action: dict: tuple(int) -> int or None
        Mapping from keys pressed to action performed.
        For example if pressed 'w' and space at the same time is supposed
        to trigger action number 2 then key_to_action dict would look like this:

            {
                # ...
                sorted(ord('w'), ord(' ')) -> 2
                # ...
            }
        If None, default key_to_action mapping for that env is used, if provided.
    """

    obs_s = env.observation_space
    #assert type(obs_s) == gym.spaces.box.Box
    #assert len(obs_s.shape) == 2 or (len(obs_s.shape) == 3 and obs_s.shape[2] in [1,3])

    if keys_to_action is None:
        if hasattr(env, 'get_keys_to_action'):
            keys_to_action = env.get_keys_to_action()
        elif hasattr(env.unwrapped, 'get_keys_to_action'):
            keys_to_action = env.unwrapped.get_keys_to_action()
    relevant_keys = set(sum(map(list, keys_to_action.keys()),[]))
    
    pressed_keys = []
    running = True
    env_done = True

    record_num = 0
    record_total = 0
    obs = env.reset()
    do_restart = False
    last_keys = []              ## Prevent overacting
    while running:
        if do_restart:
            do_restart = False
            env.reset()
            pressed_keys = []
            continue
        if len(pressed_keys) == 0:
            action = keys_to_action[()]
            with Profiler("Play Env: step"):
                start = time.time()
                obs, rew, env_done, info = env.step(action)
                record_total += time.time() - start
                record_num += 1
            #print(info['sensor'])
            print("Play mode: reward %f" % rew)
        for p_key in pressed_keys:
            action = keys_to_action[(p_key, )]
            prev_obs = obs
            with Profiler("Play Env: step"):
                start = time.time()
                obs, rew, env_done, info = env.step(action)
                record_total += time.time() - start
                record_num += 1
            print("Play mode: reward %f" % rew)
        if callback is not None:
            callback(prev_obs, obs, action, rew, env_done, info)
        # process pygame events
        key_codes = env.get_key_pressed(relevant_keys)
        #print("Key codes", key_codes)
        pressed_keys = []

        for key in key_codes:
            if key == ord('r') and key not in last_keys:
                do_restart = True
            if key == ord('j') and key not in last_keys:
                env.robot.turn_left()
            if key == ord('l') and key not in last_keys:
                env.robot.turn_right()
            if key == ord('i') and key not in last_keys:
                env.robot.move_forward()
            if key == ord('k') and key not in last_keys:
                env.robot.move_backward()
            if key not in relevant_keys:
                continue
            pressed_keys.append(key) 
            
        last_keys = key_codes
        

class PlayPlot(object):
    def __init__(self, callback, horizon_timesteps, plot_names):
        self.data_callback = callback
        self.horizon_timesteps = horizon_timesteps
        self.plot_names = plot_names

        num_plots = len(self.plot_names)
        self.fig, self.ax = plt.subplots(num_plots)
        if num_plots == 1:
            self.ax = [self.ax]
        for axis, name in zip(self.ax, plot_names):
            axis.set_title(name)
        self.t = 0
        self.cur_plot = [None for _ in range(num_plots)]
        self.data     = [deque(maxlen=horizon_timesteps) for _ in range(num_plots)]

    def callback(self, obs_t, obs_tp1, action, rew, done, info):
        points = self.data_callback(obs_t, obs_tp1, action, rew, done, info)
        for point, data_series in zip(points, self.data):
            data_series.append(point)
        self.t += 1

        xmin, xmax = max(0, self.t - self.horizon_timesteps), self.t

        for i, plot in enumerate(self.cur_plot):
            if plot is not None:
                plot.remove()
            self.cur_plot[i] = self.ax[i].scatter(range(xmin, xmax), list(self.data[i]))
            self.ax[i].set_xlim(xmin, xmax)
        plt.pause(0.000001)




import argparse
import os
import re
import torch
import torch.nn as nn
import numpy as np
from torchvision import datasets, transforms
import torch.backends.cudnn as cudnn
from torch.autograd import Variable
import torchvision.utils as vutils
from gibson.data.datasets import PairDataset
from gibson.learn.completion import CompletionNet, identity_init, Perceptual
from tensorboardX import SummaryWriter
from datetime import datetime
import gibson.learn.vision_utils
import torch.nn.functional as F
import torchvision.models as models


def weights_init(m):
    classname = m.__class__.__name__
    if classname.find('Conv') != -1:
        m.weight.data.normal_(0.0, 0.02)
    elif classname.find('Linear') != -1:
        m.weight.data.normal_(0.0, 0.02)
        m.bias.data.fill_(0)
    elif classname.find('BatchNorm') != -1:
        m.weight.data.normal_(1.0, 0.02)
        m.bias.data.fill_(0)


def crop(source, source_depth, target, zoom = 1, patchsize = 256):
    bs = source.size(0)
    if zoom > 1:
        source = F.avg_pool2d(source, zoom, zoom)
        source_depth = F.avg_pool2d(source_depth, zoom, zoom)
        target = F.avg_pool2d(target, zoom, zoom)

    source_cropped = Variable(torch.zeros(4 * bs, 3, patchsize, patchsize)).cuda()
    source_depth_cropped = Variable(torch.zeros(4 * bs, 2, patchsize, patchsize)).cuda()
    target_cropped = Variable(torch.zeros(4 * bs, 3, patchsize, patchsize)).cuda()
    p = patchsize//2
    for i in range(bs):
        for j in range(4):
            idx = i * 4 + j
            blurry_margin = 1024 // 8 // zoom
            centerx = np.random.randint(blurry_margin + p, 1024//zoom - blurry_margin - p)
            centery = np.random.randint(p, 1024//zoom * 2 - p)
            source_cropped[idx] = source[i, :, centerx - p:centerx + p, centery - p:centery + p]
            source_depth_cropped[idx] = source_depth[i, :, centerx - p:centerx + p, centery - p:centery + p]
            target_cropped[idx] = target[i, :, centerx - p:centerx + p, centery - p:centery + p]

    return source_cropped, source_depth_cropped, target_cropped


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--dataroot', required=True, help='path to dataset')
    parser.add_argument('--debug', action='store_true', help='debug mode')
    parser.add_argument('--imgsize', type=int, default=256, help='image size')
    parser.add_argument('--nf', type=int, default=64, help='number of filters')
    parser.add_argument('--batchsize', type=int, default=20, help='batchsize')
    parser.add_argument('--workers', type=int, default=9, help='number of workers')
    parser.add_argument('--nepoch', type=int, default=50, help='number of epochs')
    parser.add_argument('--lr', type=float, default=2e-5, help='learning rate, default=0.002')
    parser.add_argument('--beta1', type=float, default=0.5, help='beta1 for adam. default=0.5')
    parser.add_argument('--outf', type=str, default="filler_pano_pc_full", help='output folder')
    parser.add_argument('--model', type=str, default="", help='model path')
    parser.add_argument('--cepoch', type=int, default=0, help='current epoch')
    parser.add_argument('--loss', type=str, default="perceptual", help='l1 only')
    parser.add_argument('--init', type=str, default="iden", help='init method')
    parser.add_argument('--l1', type=float, default=0, help='add l1 loss')
    parser.add_argument('--color_coeff', type=float, default=0, help='add color match loss')
    parser.add_argument('--unfiller', action='store_true', help='debug mode')
    parser.add_argument('--joint', action='store_true', help='debug mode')
    parser.add_argument('--use_depth', action='store_true', default=False, help='debug mode')
    parser.add_argument('--zoom', type=int, default=1, help='debug mode')
    parser.add_argument('--patchsize', type=int, default=256, help='debug mode')

    mean = torch.from_numpy(np.array([0.57441127, 0.54226291, 0.50356019]).astype(np.float32)).clone()
    opt = parser.parse_args()
    print(opt)
    writer = SummaryWriter(opt.outf + '/runs/' + datetime.now().strftime('%B%d  %H:%M:%S'))
    try:
        os.makedirs(opt.outf)
    except OSError:
        pass

    zoom = opt.zoom
    patchsize = opt.patchsize

    tf = transforms.Compose([
        transforms.ToTensor(),
    ])

    mist_tf = transforms.Compose([
        transforms.ToTensor(),
    ])

    d = PairDataset(root=opt.dataroot, transform=tf, mist_transform=mist_tf)
    d_test = PairDataset(root=opt.dataroot, transform=tf, mist_transform=mist_tf, train=False)

    cudnn.benchmark = True

    dataloader = torch.utils.data.DataLoader(d, batch_size=opt.batchsize, shuffle=True, num_workers=int(opt.workers),
                                             drop_last=True, pin_memory=False)
    dataloader_test = torch.utils.data.DataLoader(d_test, batch_size=opt.batchsize, shuffle=True,
                                                  num_workers=int(opt.workers), drop_last=True, pin_memory=False)

    img = Variable(torch.zeros(opt.batchsize, 3, 1024, 2048)).cuda()
    maskv = Variable(torch.zeros(opt.batchsize, 2, 1024, 2048)).cuda()
    img_original = Variable(torch.zeros(opt.batchsize, 3, 1024, 2048)).cuda()
    label = Variable(torch.LongTensor(opt.batchsize * 4)).cuda()

    comp = CompletionNet(norm=nn.BatchNorm2d, nf=opt.nf)

    current_epoch = opt.cepoch

    comp = torch.nn.DataParallel(comp).cuda()

    if opt.init == 'iden':
        comp.apply(identity_init)
    else:
        comp.apply(weights_init)

    if opt.model != '':
        comp.load_state_dict(torch.load(opt.model))
        # dis.load_state_dict(torch.load(opt.model.replace("G", "D")))
        current_epoch = opt.cepoch

    if opt.unfiller:
        comp2 = CompletionNet(norm=nn.BatchNorm2d, nf=64)
        comp2 = torch.nn.DataParallel(comp2).cuda()
        if opt.model != '':
            comp2.load_state_dict(torch.load(opt.model.replace('G', 'G2')))
        optimizerG2 = torch.optim.Adam(comp2.parameters(), lr=opt.lr, betas=(opt.beta1, 0.999))

    l2 = nn.MSELoss()
    # if opt.loss == 'train_init':
    #    params = list(comp.parameters())
    #    sel = np.random.choice(len(params), len(params)/2, replace=False)
    #    params_sel = [params[i] for i in sel]
    #    optimizerG = torch.optim.Adam(params_sel, lr = opt.lr, betas = (opt.beta1, 0.999))
    #
    # else:
    optimizerG = torch.optim.Adam(comp.parameters(), lr=opt.lr, betas=(opt.beta1, 0.999))

    curriculum = (200000, 300000)  # step to start D training and G training, slightly different from the paper
    alpha = 0.004

    errG_data = 0
    errD_data = 0

    vgg16 = models.vgg16(pretrained=False)
    vgg16.load_state_dict(torch.load('vgg16-397923af.pth'))
    feat = vgg16.features
    p = torch.nn.DataParallel(Perceptual(feat)).cuda()

    for param in p.parameters():
        param.requires_grad = False

    imgnet_mean = torch.from_numpy(np.array([0.485, 0.456, 0.406]).astype(np.float32)).clone()
    imgnet_std = torch.from_numpy(np.array([0.229, 0.224, 0.225]).astype(np.float32)).clone()

    imgnet_mean_img = Variable(imgnet_mean.view(1, 3, 1, 1).repeat(opt.batchsize * 4, 1, patchsize, patchsize)).cuda()
    imgnet_std_img = Variable(imgnet_std.view(1, 3, 1, 1).repeat(opt.batchsize * 4, 1, patchsize, patchsize)).cuda()

    test_loader_enum = enumerate(dataloader_test)
    for epoch in range(current_epoch, opt.nepoch):
        for i, data in enumerate(dataloader, 0):
            optimizerG.zero_grad()
            source = data[0]
            source_depth = data[1]
            target = data[2]
            step = i + epoch * len(dataloader)

            mask = (torch.sum(source[:, :3, :, :], 1) > 0).float().unsqueeze(1)
            # img_mean = torch.sum(torch.sum(source[:,:3,:,:], 2),2) / torch.sum(torch.sum(mask, 2),2).view(opt.batchsize,1)

            source[:, :3, :, :] += (1 - mask.repeat(1, 3, 1, 1)) * mean.view(1, 3, 1, 1).repeat(opt.batchsize, 1, 1024,
                                                                                                2048)
            source_depth = source_depth[:, :, :, 0].unsqueeze(1)
            # print(source_depth.size(), mask.size())
            source_depth = torch.cat([source_depth, mask], 1)
            img.data.copy_(source)
            maskv.data.copy_(source_depth)
            img_original.data.copy_(target)
            imgc, maskvc, img_originalc = crop(img, maskv, img_original, zoom, patchsize)
            # from IPython import embed; embed()
            recon = comp(imgc, maskvc)

            if opt.loss == "train_init":
                loss = l2(recon, imgc[:, :3, :, :])
            elif opt.loss == 'l1':
                loss = l2(recon, img_originalc)
            elif opt.loss == 'perceptual':
                loss = l2(p(recon), p(img_originalc).detach()) + opt.l1 * l2(recon, img_originalc)
            elif opt.loss == 'color_stable':
                loss = l2(p(recon.view(recon.size(0) * 3, 1, patchsize, patchsize).repeat(1, 3, 1, 1)),
                          p(img_originalc.view(img_originalc.size(0) * 3, 1, patchsize, patchsize).repeat(1, 3, 1, 1)).detach())
            elif opt.loss == 'color_correction':
                recon_percept = p((recon - imgnet_mean_img) / imgnet_std_img)
                org_percept = p((img_originalc - imgnet_mean_img) / (imgnet_std_img)).detach()
                loss = l2(recon_percept, org_percept)
                for scale in [32]:
                    img_originalc_patch = img_originalc.view(opt.batchsize * 4, 3, patchsize // scale, scale, patchsize // scale,
                                                             scale).transpose(4, 3).contiguous().view(opt.batchsize * 4,
                                                                                                      3, patchsize // scale,
                                                                                                      patchsize // scale, -1)
                    recon_patch = recon.view(opt.batchsize * 4, 3, patchsize // scale, scale, patchsize // scale, scale).transpose(4,
                                                                                                                     3).contiguous().view(
                        opt.batchsize * 4, 3, patchsize // scale, patchsize // scale, -1)
                    img_originalc_patch_mean = img_originalc_patch.mean(dim=-1)
                    recon_patch_mean = recon_patch.mean(dim=-1)
                    # recon_patch_cov = []
                    # img_originalc_patch_cov = []

                    # for j in range(3):
                    #    recon_patch_cov.append((recon_patch * recon_patch[:,j:j+1].repeat(1,3,1,1,1)).mean(dim=-1))
                    #    img_originalc_patch_cov.append((img_originalc_patch * img_originalc_patch[:,j:j+1].repeat(1,3,1,1,1)).mean(dim=-1))

                    # recon_patch_cov_cat = torch.cat(recon_patch_cov,1)
                    # img_originalc_patch_cov_cat = torch.cat(img_originalc_patch_cov, 1)

                    color_loss = l2(recon_patch_mean,
                                    img_originalc_patch_mean)  # + l2(recon_patch_cov_cat, img_originalc_patch_cov_cat.detach())

                    loss += opt.color_coeff * color_loss

                    print("color loss %f" % color_loss.data[0])

            loss.backward(retain_graph=True)

            if opt.unfiller:
                optimizerG2.zero_grad()

                recon2 = comp2(img_originalc, maskvc)

                if not opt.joint:
                    recon2_percept = p((recon2 - imgnet_mean_img) / imgnet_std_img)
                    recon_percept = p((recon - imgnet_mean_img) / imgnet_std_img)
                    loss2 = l2(recon2_percept, recon_percept.detach())
                else:
                    recon_percept = p((recon - imgnet_mean_img) / imgnet_std_img)
                    z = Variable(torch.zeros(recon_percept.size()).cuda())
                    recon2_percept = p((recon2 - imgnet_mean_img) / imgnet_std_img)

                    loss2 = l2(recon2_percept - recon_percept, z)

                    loss2 += 0.2 * l2(recon2_percept, org_percept)

                for scale in [32]:
                    img_originalc_patch = recon.detach().view(opt.batchsize * 4, 3, patchsize / scale, scale, patchsize / scale,
                                                              scale).transpose(4, 3).contiguous().view(
                        opt.batchsize * 4, 3, patchsize / scale, patchsize / scale, -1)
                    recon2_patch = recon2.view(opt.batchsize * 4, 3, patchsize / scale, scale, patchsize / scale, scale).transpose(
                        4, 3).contiguous().view(opt.batchsize * 4, 3, patchsize / scale, patchsize / scale, -1)
                    img_originalc_patch_mean = img_originalc_patch.mean(dim=-1)
                    recon2_patch_mean = recon2_patch.mean(dim=-1)
                    recon2_patch_cov = []
                    img_originalc_patch_cov = []

                    for j in range(3):
                        recon2_patch_cov.append(
                            (recon2_patch * recon2_patch[:, j:j + 1].repeat(1, 3, 1, 1, 1)).mean(dim=-1))
                        img_originalc_patch_cov.append(
                            (img_originalc_patch * img_originalc_patch[:, j:j + 1].repeat(1, 3, 1, 1, 1)).mean(dim=-1))

                    recon2_patch_cov_cat = torch.cat(recon2_patch_cov, 1)
                    img_originalc_patch_cov_cat = torch.cat(img_originalc_patch_cov, 1)

                    z = Variable(torch.zeros(img_originalc_patch_mean.size()).cuda())
                    if opt.joint:
                        color_loss = l2(recon2_patch_mean - img_originalc_patch_mean, z)
                    else:
                        color_loss = l2(recon2_patch_mean, img_originalc_patch_mean)

                    loss2 += opt.color_coeff * color_loss

                    print("color loss %f" % color_loss.data[0])

                loss2 = loss2 * 0.3
                loss2.backward(retain_graph=True)
                print("loss2 %f" % loss2.data[0])
                optimizerG2.step()

                if i % 10 == 0:
                    writer.add_scalar('MSEloss2', loss2.data[0], step)

            # from IPython import embed; embed()
            if opt.loss == "train_init":
                for param in comp.parameters():
                    if len(param.size()) == 4:
                        # print(param.size())
                        nk = param.size()[2] // 2
                        if nk > 5:
                            param.grad[:nk, :, :, :] = 0

            optimizerG.step()

            print('[%d/%d][%d/%d] %d MSEloss: %f G_loss %f D_loss %f' % (
            epoch, opt.nepoch, i, len(dataloader), step, loss.data[0], errG_data, errD_data))

            if i % 200 == 0:

                test_i, test_data = next(test_loader_enum)
                if test_i > len(dataloader_test) - 5:
                    test_loader_enum = enumerate(dataloader_test)

                source = test_data[0]
                source_depth = test_data[1]
                target = test_data[2]

                mask = (torch.sum(source[:, :3, :, :], 1) > 0).float().unsqueeze(1)

                source[:, :3, :, :] += (1 - mask.repeat(1, 3, 1, 1)) * mean.view(1, 3, 1, 1).repeat(opt.batchsize, 1,
                                                                                                    1024, 2048)
                source_depth = source_depth[:, :, :, 0].unsqueeze(1)
                source_depth = torch.cat([source_depth, mask], 1)
                img.data.copy_(source)
                maskv.data.copy_(source_depth)
                img_original.data.copy_(target)
                imgc, maskvc, img_originalc = crop(img, maskv, img_original, zoom, patchsize)
                comp.eval()
                recon = comp(imgc, maskvc)
                comp.train()

                if opt.unfiller:
                    comp2.eval()
                    # maskvc.data.fill_(0)
                    recon2 = comp2(img_originalc, maskvc)
                    comp2.train()
                    visual = torch.cat([imgc.data[:, :3, :, :], recon.data, recon2.data, img_originalc.data], 3)
                else:
                    visual = torch.cat([imgc.data[:, :3, :, :], recon.data, img_originalc.data], 3)

                visual = vutils.make_grid(visual, normalize=True)
                writer.add_image('image', visual, step)
                vutils.save_image(visual, '%s/compare%d_%d.png' % (opt.outf, epoch, i), nrow=1)

            if i % 10 == 0:
                writer.add_scalar('MSEloss', loss.data[0], step)
                writer.add_scalar('G_loss', errG_data, step)
                writer.add_scalar('D_loss', errD_data, step)

            if i % 2000 == 0:
                torch.save(comp.state_dict(), '%s/compG_epoch%d_%d.pth' % (opt.outf, epoch, i))

                if opt.unfiller:
                    torch.save(comp2.state_dict(), '%s/compG2_epoch%d_%d.pth' % (opt.outf, epoch, i))


if __name__ == '__main__':
    main()



from __future__ import print_function

import torch.backends.cudnn as cudnn
import torch.nn as nn
import torch.optim as optim
import torch
import torchvision
from torchvision import datasets, transforms
from torch.autograd import Variable
import torch.nn.functional as F
import shutil
import time

cudnn.benchmark = True


class AdaptiveNorm2d(nn.Module):
    def __init__(self, nchannel, momentum=0.05):
        super(AdaptiveNorm2d, self).__init__()
        self.nm = nn.BatchNorm2d(nchannel, momentum=momentum)
        self.w0 = nn.Parameter(torch.zeros(1))
        self.w1 = nn.Parameter(torch.ones(1))

    def forward(self, x):
        return self.w0.repeat(x.size()) * self.nm(x) + self.w1.repeat(x.size()) * x


class CompletionNet(nn.Module):
    def __init__(self, norm=AdaptiveNorm2d, nf=64, skip_first_bn=False):
        super(CompletionNet, self).__init__()

        self.nf = nf
        alpha = 0.05
        if skip_first_bn:
            self.convs = nn.Sequential(
                nn.Conv2d(5, nf // 4, kernel_size=5, stride=1, padding=2),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf // 4, nf, kernel_size=5, stride=2, padding=2),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf, kernel_size=3, stride=1, padding=2),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf * 4, kernel_size=5, stride=2, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=2, padding=2),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=4, padding=4),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=8, padding=8),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=16, padding=16),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=32, padding=32),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.ConvTranspose2d(nf * 4, nf, kernel_size=4, stride=2, padding=1),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf, kernel_size=3, stride=1, padding=1),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.ConvTranspose2d(nf, nf // 4, kernel_size=4, stride=2, padding=1),
                norm(nf // 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf // 4, nf // 4, kernel_size=3, stride=1, padding=1),
                norm(nf // 4, momentum=alpha),
                nn.LeakyReLU(0.1),
                nn.Conv2d(nf // 4, 3, kernel_size=3, stride=1, padding=1),
            )
        else:
            self.convs = nn.Sequential(
                nn.Conv2d(5, nf // 4, kernel_size=5, stride=1, padding=2),
                norm(nf//4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf // 4, nf, kernel_size=5, stride=2, padding=2),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf, kernel_size=3, stride=1, padding=2),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf * 4, kernel_size=5, stride=2, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=2, padding=2),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=4, padding=4),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=8, padding=8),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=16, padding=16),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, dilation=32, padding=32),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf * 4, nf * 4, kernel_size=3, stride=1, padding=1),
                norm(nf * 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.ConvTranspose2d(nf * 4, nf, kernel_size=4, stride=2, padding=1),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf, nf, kernel_size=3, stride=1, padding=1),
                norm(nf, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.ConvTranspose2d(nf, nf // 4, kernel_size=4, stride=2, padding=1),
                norm(nf // 4, momentum=alpha),
                nn.LeakyReLU(0.1),

                nn.Conv2d(nf // 4, nf // 4, kernel_size=3, stride=1, padding=1),
                norm(nf // 4, momentum=alpha),
                nn.LeakyReLU(0.1),
                nn.Conv2d(nf // 4, 3, kernel_size=3, stride=1, padding=1),
            )

    def forward(self, x, mask):
        return F.tanh(self.convs(torch.cat([x, mask], 1)))


def identity_init(m):
    classname = m.__class__.__name__
    if classname.find('Conv2d') != -1:
        m.weight.data.fill_(0)
        o, i, k1, k2 = m.weight.data.size()
        cx, cy = k1 // 2, k2 // 2
        nc = min(o, i)
        print(nc)
        for i in range(nc):
            m.weight.data[i, i, cx, cy] = 1
        m.bias.data.fill_(0)

        if m.stride[0] == 2:
            for i in range(nc):
                m.weight.data[i + nc, i, cx + 1, cy] = 1
                m.weight.data[i + nc * 2, i, cx, cy + 1] = 1
                m.weight.data[i + nc * 3, i, cx + 1, cy + 1] = 1

    elif classname.find('ConvTranspose2d') != -1:
        o, i, k1, k2 = m.weight.data.size()
        nc = min(o, i)
        cx, cy = k1 // 2 - 1, k2 // 2 - 1
        m.weight.data.fill_(0)
        for i in range(nc):
            m.weight.data[i, i, cx, cy] = 1
            m.weight.data[i + nc, i, cx + 1, cy] = 1
            m.weight.data[i + nc * 2, i, cx, cy + 1] = 1
            m.weight.data[i + nc * 3, i, cx + 1, cy + 1] = 1
        m.bias.data.fill_(0)

    elif classname.find('BatchNorm') != -1:
        m.weight.data.fill_(1)
        m.bias.data.fill_(0)


class Perceptual(nn.Module):
    def __init__(self, features):
        super(Perceptual, self).__init__()
        self.features = features

    def forward(self, x):
        bs = x.size(0)
        x = self.features[0](x)
        x = self.features[1](x)
        x = self.features[2](x)
        x0 = x.view(bs, -1, 1)
        x = F.relu(x)
        x = self.features[4](x)
        x = self.features[5](x)
        x = self.features[6](x)
        x = self.features[7](x)
        x1 = x.view(bs, -1, 1)
        x = F.relu(x)
        x = self.features[9](x)
        x = self.features[10](x)
        x = self.features[11](x)
        x = self.features[12](x)
        x2 = x.view(bs, -1, 1)
        x = F.relu(x)
        x = self.features[14](x)
        x = self.features[15](x)
        x = self.features[16](x)
        x = self.features[17](x)
        x = self.features[18](x)
        x = self.features[19](x)
        x3 = x.view(bs, -1, 1)
        x = F.relu(x)
        x = self.features[21](x)
        x = self.features[22](x)
        x = self.features[23](x)
        x = self.features[24](x)
        x = self.features[25](x)
        x = self.features[26](x)
        x4 = x.view(bs, -1, 1)

        perfeat = torch.cat([x0, x1, x2, x3, x4], 1)

        return perfeat



__all__ = ['Monitor', 'get_monitor_files', 'load_results']

import gym
from gym.core import Wrapper
import time
from glob import glob
import csv
import os.path as osp
import json

class Monitor(Wrapper):
    EXT = "monitor.csv"
    f = None

    def __init__(self, env, filename, allow_early_resets=False, reset_keywords=()):
        Wrapper.__init__(self, env=env)
        self.tstart = time.time()
        if filename is None:
            self.f = None
            self.logger = None
        else:
            if not filename.endswith(Monitor.EXT):
                if osp.isdir(filename):
                    filename = osp.join(filename, Monitor.EXT)
                else:
                    filename = filename + "." + Monitor.EXT
            self.f = open(filename, "wt")
            self.f.write('#%s\n'%json.dumps({"t_start": self.tstart, "gym_version": gym.__version__,
                "env_id": env.spec.id if env.spec else 'Unknown'}))
            self.logger = csv.DictWriter(self.f, fieldnames=('r', 'l', 't')+reset_keywords)
            self.logger.writeheader()

        self.reset_keywords = reset_keywords
        self.allow_early_resets = allow_early_resets
        self.rewards = None
        self.needs_reset = True
        self.episode_rewards = []
        self.episode_lengths = []
        self.total_steps = 0
        self.current_reset_info = {} # extra info about the current episode, that was passed in during reset()

        ## Cambria specific
        self.sensor_space = env.sensor_space


    def _reset(self, **kwargs):
        if not self.allow_early_resets and not self.needs_reset:
            raise RuntimeError("Tried to reset an environment before done. If you want to allow early resets, wrap your env with Monitor(env, path, allow_early_resets=True)")
        self.rewards = []
        self.needs_reset = False
        for k in self.reset_keywords:
            v = kwargs.get(k)
            if v is None:
                raise ValueError('Expected you to pass kwarg %s into reset'%k)
            self.current_reset_info[k] = v
        return self.env.reset(**kwargs)

    def _step(self, action):
        if self.needs_reset:
            raise RuntimeError("Tried to step environment that needs reset")
        ob, rew, done, info = self.env.step(action)
        self.rewards.append(rew)
        if done:
            self.needs_reset = True
            eprew = sum(self.rewards)
            eplen = len(self.rewards)
            epinfo = {"r": round(eprew, 6), "l": eplen, "t": round(time.time() - self.tstart, 6)}
            epinfo.update(self.current_reset_info)
            if self.logger:
                self.logger.writerow(epinfo)
                self.f.flush()
            self.episode_rewards.append(eprew)
            self.episode_lengths.append(eplen)
            info['episode'] = epinfo
        self.total_steps += 1
        return (ob, rew, done, info)

    def close(self):
        if self.f is not None:
            self.f.close()

    def get_total_steps(self):
        return self.total_steps

    def get_episode_rewards(self):
        return self.episode_rewards

    def get_episode_lengths(self):
        return self.episode_lengths

class LoadMonitorResultsError(Exception):
    pass

def get_monitor_files(dir):
    return glob(osp.join(dir, "*" + Monitor.EXT))

def load_results(dir):
    import pandas
    monitor_files = glob(osp.join(dir, "*monitor.*")) # get both csv and (old) json files
    if not monitor_files:
        raise LoadMonitorResultsError("no monitor files of the form *%s found in %s" % (Monitor.EXT, dir))
    dfs = []
    headers = []
    for fname in monitor_files:
        with open(fname, 'rt') as fh:
            if fname.endswith('csv'):
                firstline = fh.readline()
                assert firstline[0] == '#'
                header = json.loads(firstline[1:])
                df = pandas.read_csv(fh, index_col=None)
                headers.append(header)
            elif fname.endswith('json'): # Deprecated json format
                episodes = []
                lines = fh.readlines()
                header = json.loads(lines[0])
                headers.append(header)
                for line in lines[1:]:
                    episode = json.loads(line)
                    episodes.append(episode)
                df = pandas.DataFrame(episodes)
        df['t'] += header['t_start']
        dfs.append(df)
    df = pandas.concat(dfs)
    df.sort_values('t', inplace=True)
    df['t'] -= min(header['t_start'] for header in headers)
    df.headers = headers # HACK to preserve backwards compatibility
    return df


from __future__ import print_function
import torch.utils.data as data
from PIL import Image
import os, time
import os.path
from multiprocessing import Pool
from functools import partial

from gibson.assets.assets_manager import AssetsManager
from gibson.core.render.profiler import Profiler
import errno
import torch
import json
import codecs
import cv2
import numpy as np
import ctypes as ct
import sys
from tqdm import *
import torchvision.transforms as transforms
import argparse
import json
from numpy.linalg import inv
import pickle
from gibson import assets

IMG_EXTENSIONS = [
    '.jpg', '.JPG', '.jpeg', '.JPEG',
    '.png', '.PNG', '.ppm', '.PPM', '.bmp', '.BMP',
]


def is_image_file(filename):
    return any(filename.endswith(extension) for extension in IMG_EXTENSIONS)


def default_loader(path):
    ## Heavy usage
    img = cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)#.convert('RGB')
    #img = Image.open(path)
    return img


def depth_loader(path):
    ## Heavy usage
    ## TODO: Image.open for depth image is main data loading bottleneck
    #img = cv2.imread(path, cv2.IMREAD_GRAYSCALE)#.convert('I')
    img = Image.open(path)
    return img


def get_model_path(model_id):
    data_path = os.path.join(AssetsManager().get_assets_path(), 'dataset')
    assert (model_id in os.listdir(data_path)) or model_id == 'stadium', "Model {} does not exist".format(model_id)
    return os.path.join(data_path, model_id)

def get_item_fn(inds, select, root, loader, transform, off_3d, target_transform, depth_trans, off_pc_render, dll, train, require_rgb):
    """ Functional programming version of Dataset.__getitem__
    The advantage is that it is pickle-friendly and supports python multiprocessing
    
    Argument: 
        inds: tuple of scene index and output index
    """
    index, out_i = inds
    scene = select[index][0][0]
    uuids = [item[1] for item in select[index]]
    paths = ([os.path.join(root, scene, 'pano', 'rgb', "point_" + item + "_view_equirectangular_domain_rgb.png") for item in uuids])
    mist_paths = ([os.path.join(root, scene, 'pano', 'mist', "point_" + item + "_view_equirectangular_domain_mist.png") for item in uuids])
    normal_paths = ([os.path.join(root, scene, 'pano', 'normal', "point_" + item + "_view_equirectangular_domain_normal.png") for item in uuids])
    pose_paths = ([os.path.join(root, scene, 'pano', 'points', "point_" + item + ".json") for item in uuids])
    semantic_paths = ([os.path.join(root, scene, 'pano', 'semantic', "point_" + item + "_view_equirectangular_domain_semantic.png") for item in uuids])
    poses = []
    for i, item in enumerate(pose_paths):
        f = open(item)
        pose_dict = json.load(f)
        p = np.concatenate(np.array(pose_dict[1][u'camera_rt_matrix'])).astype(np.float32).reshape((4, 4))
        rotation = np.array([[0, 1, 0, 0], [0, 0, 1, 0], [-1, 0, 0, 0], [0, 0, 0, 1]])
        p = np.dot(p, rotation)
        poses.append(p)
        f.close()

    img_paths = paths[1:]
    target_path = paths[0]
    img_poses = poses[1:]
    target_pose = poses[0]

    mist_img_paths = mist_paths[1:]
    mist_target_path = mist_paths[0]

    normal_img_paths = normal_paths[1:]
    normal_target_path = normal_paths[0]

    semantic_img_paths = semantic_paths[1:]
    semantic_target_path = semantic_paths[0]
    poses_relative = []

    semantic_imgs = None
    semantic_target = None

    normal_imgs = None
    normal_target = None

    mist_imgs = None
    mist_target = None

    imgs = None
    target = None

    for pose_i, item in enumerate(img_poses):
        pose_i = pose_i + 1
        relative = np.dot(inv(target_pose), item)
        poses_relative.append(torch.from_numpy(relative))
    
    if require_rgb:
        imgs = [loader(item) for item in img_paths]
        target = loader(target_path)
        org_img = imgs[0].copy()

    if not off_3d and require_rgb:
        mist_imgs = [depth_loader(item) for item in mist_img_paths]
        mist_target = depth_loader(mist_target_path)
        if train:
            normal_imgs = [loader(item) for item in normal_img_paths]
            normal_target = loader(normal_target_path)


    if not transform is None and require_rgb:
        imgs = [transform(item) for item in imgs]
    if not target_transform is None and require_rgb:
        target = target_transform(target)

    if not off_3d and require_rgb:
        mist_imgs = [np.expand_dims(np.array(item).astype(np.float32) / 65536.0, 2) for item in mist_imgs]
        org_mist = mist_imgs[0][:, :, 0].copy()
        mist_target = np.expand_dims(np.array(mist_target).astype(np.float32) / 65536.0, 2)

        if not depth_trans is None:
            mist_imgs = [depth_trans(item) for item in mist_imgs]
        if not depth_trans is None:
            mist_target = depth_trans(mist_target)

        if train:
            if not transform is None:
                normal_imgs = [transform(item) for item in normal_imgs]
            if not target_transform is None:
                normal_target = target_transform(normal_target)

    if not off_pc_render and require_rgb:
        img = np.array(org_img)
        h, w, _ = img.shape
        render = np.zeros((h, w, 3), dtype='uint8')
        target_depth = np.zeros((h, w)).astype(np.float32)
        depth = org_mist
        pose = poses_relative[0].numpy()
        dll.render(ct.c_int(img.shape[0]),
                        ct.c_int(img.shape[1]),
                        img.ctypes.data_as(ct.c_void_p),
                        depth.ctypes.data_as(ct.c_void_p),
                        pose.ctypes.data_as(ct.c_void_p),
                        render.ctypes.data_as(ct.c_void_p),
                        target_depth.ctypes.data_as(ct.c_void_p)
                        )
        if not transform is None:
            render = transform(Image.fromarray(render))
        if not depth_trans is None:
            target_depth = depth_trans(np.expand_dims(target_depth, 2))

    if off_3d:
        out = (imgs, target, poses_relative)
    elif off_pc_render:
        out = (imgs, target, mist_imgs, mist_target, normal_imgs, normal_target, poses_relative)
    else:
        out = (imgs, target, mist_imgs, mist_target, normal_imgs, normal_target, poses_relative, render, target_depth)
    return (out_i, out)


class ViewDataSet3D(data.Dataset):
    def __init__(self, root=None, train=False, transform=None, mist_transform=None, loader=default_loader, seqlen=5,
                 debug=False, dist_filter=None, off_3d=True, off_pc_render=True, overwrite_fofn=False,
                 semantic_transform=np.array, env = None, only_load = None):
        print('Processing the data:')
        if not root:
            self.root = os.path.join(AssetsManager().get_assets_path(), "dataset")
        else:
            self.root = root
        self.train = train
        self.env = env
        self.loader = loader
        self.seqlen = seqlen
        self.transform = transform
        self.target_transform = transform
        self.depth_trans = mist_transform
        self.semantic_trans = semantic_transform
        self._require_semantics = "SEMANTICS" in self.env.config["ui_components"]
        self._require_rgb = "RGB_FILLED" in self.env.config["ui_components"] or "RGB_PREFILLED" in self.env.config["ui_components"] or "rgb_filled" in self.env.config["output"] or "rgb_prefill" in self.env.config["output"]
        self.off_3d = off_3d
        self.select = []
        self.fofn = self.root + '_fofn' + str(int(train)) + '.pkl'
        self.off_pc_render = off_pc_render
        self.dll = None

        if not self.off_pc_render:
            self.dll = np.ctypeslib.load_library('render', '.')

        if overwrite_fofn or not os.path.isfile(self.fofn):

            if only_load is None:
                self.scenes = sorted([d for d in (os.listdir(self.root)) if
                                  os.path.isdir(os.path.join(self.root, d)) and os.path.isfile(
                                      os.path.join(self.root, d, 'camera_poses.csv')) and os.path.isdir(
                                      os.path.join(self.root, d, 'pano'))])

                num_scenes = len(self.scenes)
                num_train = int(num_scenes * 0.9)

            else:

                self.scenes = sorted([only_load])
                num_scenes = 1
                num_train = 0


            print("Total %d scenes %d train %d test" % (num_scenes, num_train, num_scenes - num_train))
            if train:
                self.scenes = self.scenes[:num_train]

            self.meta = {}

            last = len(self.scenes)

            for scene in self.scenes[:last]:
                posefile = os.path.join(self.root, scene, 'camera_poses.csv')
                with open(posefile) as f:
                    for line in f:
                        l = line.strip().split(',')
                        uuid = l[0]
                        xyz = list(map(float, l[1:4]))
                        quat = list(map(float, l[4:8]))
                        if not scene in self.meta:
                            self.meta[scene] = {}
                        metadata = (uuid, xyz, quat)
                        # print(uuid, xyz)
                        if os.path.isfile(os.path.join(self.root, scene, 'pano', 'points', 'point_' + uuid + '.json')):
                            if np.linalg.norm( np.array(xyz) - np.array([0,0,0])) > 1e-5: #remove scans that are not registered
                                self.meta[scene][uuid] = metadata
            print("Indexing")

            for scene, meta in tqdm(list(self.meta.items())):
                if len(meta) < self.seqlen:
                    continue
                for uuid, v in list(meta.items()):
                    dist_list = [(uuid2, np.linalg.norm(np.array(v2[1]) - np.array(v[1]))) for uuid2, v2 in list(meta.items())]
                    dist_list = sorted(dist_list, key=lambda x: x[-1])

                    if not dist_filter is None:
                        if dist_list[1][-1] < dist_filter:
                            self.select.append([[scene, dist_list[i][0], dist_list[i][1]] for i in range(self.seqlen)])

                    else:
                        self.select.append([[scene, dist_list[i][0], dist_list[i][1]] for i in range(self.seqlen)])

            with open(self.fofn, 'wb') as fp:
                pickle.dump([self.scenes, self.meta, self.select, num_scenes, num_train], fp)

        else:
            with open(self.fofn, 'rb') as fp:
                self.scenes, self.meta, self.select, num_scenes, num_train = pickle.load(fp)
                print("Total %d scenes %d train %d test" % (num_scenes, num_train, num_scenes - num_train))


    def get_scene_info(self, index):
        scene = self.scenes[index]
        data = [(i, item) for i, item in enumerate(self.select) if item[0][0] == scene]
        uuids = ([(item[1][0][1], item[0]) for item in data])
        
        pose_paths = (
        [os.path.join(self.root, scene, 'pano', 'points', "point_" + item[0] + ".json") for item in uuids])
        poses = []
        for item in pose_paths:
            f = open(item)
            pose_dict = json.load(f)
            p = np.concatenate(np.array(pose_dict[1][u'camera_rt_matrix'])).astype(np.float32).reshape((4, 4))

            rotation = np.array([[0, 1, 0, 0], [0, 0, 1, 0], [-1, 0, 0, 0], [0, 0, 0, 1]])

            p = np.dot(p, rotation)
            poses.append(p)
            f.close()

        return uuids, poses

    def __getitem__(self, index):
        scene = self.select[index][0][0]
        uuids = [item[1] for item in self.select[index]]
        paths = (
        [os.path.join(self.root, scene, 'pano', 'rgb', "point_" + item + "_view_equirectangular_domain_rgb.png") for
         item in uuids])
        mist_paths = (
        [os.path.join(self.root, scene, 'pano', 'mist', "point_" + item + "_view_equirectangular_domain_mist.png") for
         item in uuids])
        normal_paths = (
        [os.path.join(self.root, scene, 'pano', 'normal', "point_" + item + "_view_equirectangular_domain_normal.png")
         for item in uuids])
        pose_paths = ([os.path.join(self.root, scene, 'pano', 'points', "point_" + item + ".json") for item in uuids])
        semantic_paths = ([os.path.join(self.root, scene, 'pano', 'semantic',
                                        "point_" + item + "_view_equirectangular_domain_semantic.png") for item in
                           uuids])
        poses = []
        for i, item in enumerate(pose_paths):
            f = open(item)
            pose_dict = json.load(f)
            p = np.concatenate(np.array(pose_dict[1][u'camera_rt_matrix'])).astype(np.float32).reshape((4, 4))
            rotation = np.array([[0, 1, 0, 0], [0, 0, 1, 0], [-1, 0, 0, 0], [0, 0, 0, 1]])
            p = np.dot(p, rotation)
            poses.append(p)
            f.close()

        img_paths = paths[1:]
        target_path = paths[0]
        img_poses = poses[1:]
        target_pose = poses[0]

        mist_img_paths = mist_paths[1:]
        mist_target_path = mist_paths[0]

        normal_img_paths = normal_paths[1:]
        normal_target_path = normal_paths[0]

        semantic_img_paths = semantic_paths[1:]
        semantic_target_path = semantic_paths[0]
        poses_relative = []

        semantic_imgs = None
        semantic_target = None

        normal_imgs = None
        normal_target = None

        mist_imgs = None
        mist_target = None

        imgs, target = None, None

        for pose_i, item in enumerate(img_poses):
            pose_i = pose_i + 1
            relative = np.dot(inv(target_pose), item)
            poses_relative.append(torch.from_numpy(relative))

        if self._require_rgb:
            imgs = [self.loader(item) for item in img_paths]
            target = self.loader(target_path)

        if not self.off_3d and self._require_rgb:
            mist_imgs = [depth_loader(item) for item in mist_img_paths]
            mist_target = depth_loader(mist_target_path)
            if self.train:      # Optimize
                normal_imgs = [self.loader(item) for item in normal_img_paths]
                normal_target = self.loader(normal_target_path)

        if not self.off_pc_render and self._require_rgb:
            org_img = imgs[0].copy()
            if not self.transform is None:
                imgs = [self.transform(item) for item in imgs]
            if not self.target_transform is None:
                target = self.target_transform(target)

        if not self.off_3d and self._require_rgb:
            mist_imgs = [np.expand_dims(np.array(item).astype(np.float32) / 65536.0, 2) for item in mist_imgs]
            if not self.off_pc_render:
                org_mist = mist_imgs[0][:, :, 0].copy()
            mist_target = np.expand_dims(np.array(mist_target).astype(np.float32) / 65536.0, 2)

            if not self.depth_trans is None:
                mist_imgs = [self.depth_trans(item) for item in mist_imgs]
            if not self.depth_trans is None:
                mist_target = self.depth_trans(mist_target)

            if self.train:
                if not self.transform is None:
                    normal_imgs = [self.transform(item) for item in normal_imgs]
                if not self.target_transform is None:
                    normal_target = self.target_transform(normal_target)

        if not self.off_pc_render and self._require_rgb:
            img = np.array(org_img)
            h, w, _ = img.shape
            render = np.zeros((h, w, 3), dtype='uint8')
            target_depth = np.zeros((h, w)).astype(np.float32)
            depth = org_mist
            pose = poses_relative[0].numpy()
            self.dll.render(ct.c_int(img.shape[0]),
                            ct.c_int(img.shape[1]),
                            img.ctypes.data_as(ct.c_void_p),
                            depth.ctypes.data_as(ct.c_void_p),
                            pose.ctypes.data_as(ct.c_void_p),
                            render.ctypes.data_as(ct.c_void_p),
                            target_depth.ctypes.data_as(ct.c_void_p)
                            )
            if not self.transform is None:
                render = self.transform(Image.fromarray(render))
            if not self.depth_trans is None:
                target_depth = self.depth_trans(np.expand_dims(target_depth, 2))

        if self.off_3d:
            return imgs, target, poses_relative
        elif self.off_pc_render:
            return imgs, target, mist_imgs, mist_target, normal_imgs, normal_target, poses_relative
        else:
            return imgs, target, mist_imgs, mist_target, normal_imgs, normal_target, poses_relative, render, target_depth

    def get_multi_index(self, uuids):
        indices = range(len(uuids))
        p = Pool(16)
        partial_fn = partial(get_item_fn, select=self.select, root=self.root, loader=self.loader, transform=self.transform, off_3d=self.off_3d, target_transform=self.target_transform, depth_trans=self.depth_trans, off_pc_render=self.off_pc_render, dll=self.dll, train=self.train, require_rgb=self._require_rgb)
        mapped_pairs = list(tqdm(p.imap(partial_fn, list(zip(uuids, indices))), total=len(uuids)))
        sorted_pairs = sorted(mapped_pairs, key=lambda x: x[0])
        out_data = [key_pair[1] for key_pair in sorted_pairs]
        p.close()
        p.join()
        return out_data

    def __len__(self):
        return len(self.select)


########### BELOW THIS POINT: Legacy code #################
########### KEEPING ONLY FOR REFERENCE ####################


class Places365Dataset(data.Dataset):
    def __init__(self, root, train=True, transform=None, loader=default_loader):
        self.root = root.rstrip('/')
        self.train = train
        self.fns = []
        self.fofn = os.path.basename(root) + '_fofn' + str(int(train)) + '.pkl'
        self.loader = loader
        self.transform = transform
        if not os.path.isfile(self.fofn):
            for subdir, dirs, files in os.walk(self.root):
                if self.train:
                    files = files[:len(files) / 10 * 9]
                else:
                    files = files[len(files) / 10 * 9:]
                print(subdir)
                for file in files:
                    self.fns.append(os.path.join(subdir, file))
            with open(self.fofn, 'wb') as fp:
                pickle.dump(self.fns, fp)
        else:
            with open(self.fofn, 'rb') as fp:
                self.fns = pickle.load(fp)

    def __len__(self):
        return len(self.fns)

    def __getitem__(self, index):
        path = self.fns[index]
        img = self.loader(path)
        if not self.transform is None:
            img = self.transform(img)
        return img


class PairDataset(data.Dataset):
    def __init__(self, root, train=True, transform=None, mist_transform=None, loader=np.load):
        self.root = root.rstrip('/')
        self.train = train
        self.fns = []
        self.fofn = os.path.basename(root) + '_fofn' + str(int(train)) + '.pkl'
        self.loader = loader
        self.transform = transform
        self.mist_transform = mist_transform
        if not os.path.isfile(self.fofn):
            for subdir, dirs, files in os.walk(self.root):
                if self.train:
                    files = files[:len(files) / 10 * 9]
                else:
                    files = files[len(files) / 10 * 9:]
                print(subdir)
                for file in files:
                    if file[-3:] == 'npz':
                        self.fns.append(os.path.join(subdir, file))
            with open(self.fofn, 'wb') as fp:
                pickle.dump(self.fns, fp)
        else:
            with open(self.fofn, 'rb') as fp:
                self.fns = pickle.load(fp)

    def __len__(self):
        return len(self.fns)

    def __getitem__(self, index):
        path = self.fns[index]
        data = self.loader(path)

        try:
            source, depth, target = data['source'], data['depth'], data['target']
            # print(source.shape, depth.shape, target.shape)
        except:
            source = np.zeros((1024, 2048, 3)).astype(np.uint8)
            target = np.zeros((1024, 2048, 3)).astype(np.uint8)
            depth = np.zeros((1024, 2048)).astype(np.float32)

        if not self.transform is None:
            source = self.transform(source)
            target = self.transform(target)
            # depth = self.mist_transform(depth)
            depth = torch.from_numpy(depth.astype(np.float32))
        return source, depth, target


if __name__ == '__main__':
    print('test')
    parser = argparse.ArgumentParser()
    parser.add_argument('--debug', action='store_true', help='debug mode')
    parser.add_argument('--dataroot', required=True, help='path to dataset')
    parser.add_argument('--dataset', required=True, help='dataset type')
    opt = parser.parse_args()

    if opt.dataset == 'view3d':
        d = ViewDataSet3D(root=opt.dataroot, debug=opt.debug, seqlen=2, dist_filter=0.8, off_3d=False,
                          off_pc_render=False)
        print(len(d))
        sample = (d[1])
        print(sample)
        if sample is not None:
            print('3d test passed')

        uuids, xyzs, poses = d.get_scene_info(0)
        print(uuids, xyzs, poses)

    elif opt.dataset == 'places365':
        d = Places365Dataset(root=opt.dataroot)
        print(len(d))
        sample = d[0]
        print(sample)
        if sample is not None:
            print('places 365 test passed')

    elif opt.dataset == 'pair':
        d = PairDataset(root=opt.dataroot)
        print(len(d))
        sample = d[0]
        print(sample)
        if sample is not None:
            print('pair test passed')



import numpy as np
import ctypes as ct
import cv2
import sys
import argparse
from gibson.data.datasets import ViewDataSet3D
import torch
from torchvision import datasets, transforms
from torch.autograd import Variable
import time
from numpy import cos, sin
import matplotlib.pyplot as plt
from PIL import Image
import os
import time
from multiprocessing import Pool, cpu_count
from scipy.signal import convolve2d
from scipy.interpolate import griddata
import scipy
import torch.nn.functional as F
from torchvision import transforms


dll=np.ctypeslib.load_library('../core/render/render_cuda_f','.')

# In[6]:

def render(imgs, depths, pose, poses, tdepth):
    global fps
    t0 = time.time()
    showsz = imgs[0].shape[0]
    nimgs = len(imgs)
    show=np.zeros((showsz,showsz * 2,3),dtype='uint8')
    target_depth = (128 * tdepth[:,:,0]).astype(np.float32)

    imgs = np.array(imgs)
    depths = np.array(depths).flatten()

    rpose = np.eye(4).astype(np.float32)
    rpose[0,-1] = 1
    rpose[1,-1] = 2
    rpose[2,-1] = 1    
    
    pose_after = [rpose.dot(poses[i]).astype(np.float32) for i in range(len(imgs))]
    pose_after = np.array(pose_after)


    dll.render(ct.c_int(len(imgs)),
               ct.c_int(imgs[0].shape[0]),
               ct.c_int(imgs[0].shape[1]),
               ct.c_int(1),
               ct.c_int(1),
               imgs.ctypes.data_as(ct.c_void_p),
               depths.ctypes.data_as(ct.c_void_p),
               pose_after.ctypes.data_as(ct.c_void_p),
               show.ctypes.data_as(ct.c_void_p),
               target_depth.ctypes.data_as(ct.c_void_p)
              )

    return show, target_depth

# In[7]:

def generate_data(args):

    idx  = args[0]
    print(idx)
    d    = args[1]
    outf = args[2]
    filename = "%s/data_%d.npz" % (outf, idx)
    if not os.path.isfile(filename):
        print(idx)
        data = d[idx]   ## This operation stalls 95% of the time, CPU heavy
        sources = data[0]
        target = data[1]
        source_depths = data[2]
        target_depth = data[3]
        #target_normal = data[5]
        poses = [item.numpy() for item in data[-1]]

        show, _ =  render(sources, source_depths, poses[0], poses, target_depth)
        print(show.shape)

        Image.fromarray(show).save('%s/show%d.png' % (outf, idx))
        Image.fromarray(target).save('%s/target%d.png' % (outf, idx))

        np.savez(file = filename, source = show, depth = target_depth, target = target)

    return


parser = argparse.ArgumentParser()
parser.add_argument('--debug'  , action='store_true', help='debug mode')
parser.add_argument('--dataroot'  , required = True, help='dataset path')
parser.add_argument('--outf'  , type = str, default = '', help='path of output folder')
opt = parser.parse_args()


d = ViewDataSet3D(root=opt.dataroot, transform = np.array, mist_transform = np.array, seqlen = 5, off_3d = False, train = False)
print(len(d))

p = Pool(10)
p.map(generate_data, [(idx, d, opt.outf) for idx in range(len(d))])

#for i in range(len(d)):
#    filename = "%s/data_%d.npz" % (opt.outf, i)
#    print(filename)
#    if not os.path.isfile(filename):
#        generate_data([i, d, opt.outf])




## Usage: random_husky.py debugging
import os
from gibson.data.datasets import get_model_path


def run_depth_render():
    model_path = get_model_path()[0]
    dr_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'depth_render')
    os.chdir(dr_path)
    print("./depth_render --modelpath {}".format(model_path))
    os.system("./depth_render --modelpath {}".format(model_path))